/*
   BTN_MAI.CPP
   Main form handling functions.
   Copyright (c) 1999-2002 Bachmann Software and Services
   Author: Glenn Bachmann
*/

// system headers
#include <PalmOS.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "buttons.h"
#include "btn_res.h"

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);

void ResetForm (FormPtr formP);
Int16 PalmMessageBox ( Int16 wAlertID, char * pMessage );

static Int16 s_Age = 35;

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      
      case ctlRepeatEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      case ctlSelectEvent:
      {
         // A control button was tapped and released
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         // main form is opening

         FormPtr formP = FrmGetActiveForm ();

         MainFormInit (formP);

         FrmDrawForm (formP);

         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr formP)
{
   // set the controls to an initial state
   ResetForm ( formP );
}


void
ResetForm (FormPtr formP)
{
   Int16 wIDContactMe = FrmGetObjectIndex (formP, MainContactMeCheckbox );
   ControlPtr pCtlContactMe = (ControlPtr) FrmGetObjectPtr (formP, wIDContactMe);

   Int16 wIDAge = FrmGetObjectIndex (formP, MainAgeLabel );
   ControlPtr pCtlAge = (ControlPtr) FrmGetObjectPtr (formP, wIDAge);

   Int16 wIDSingle = FrmGetObjectIndex (formP, MainSinglePushButton );
   ControlPtr pCtlSingle = (ControlPtr) FrmGetObjectPtr (formP, wIDSingle);

   Int16 wIDMarried = FrmGetObjectIndex (formP, MainMarriedPushButton );
   ControlPtr pCtlMarried = (ControlPtr) FrmGetObjectPtr (formP, wIDMarried);

   Int16 wIDDivorced = FrmGetObjectIndex (formP, MainDivorcedPushButton );
   ControlPtr pCtlDivorced = (ControlPtr) FrmGetObjectPtr (formP, wIDDivorced);

   Int16 wIDDateofBirth = FrmGetObjectIndex (formP, MainDateofBirthSelTrigger);
   ControlPtr pCtlDateofBirth = (ControlPtr) FrmGetObjectPtr (formP, wIDDateofBirth);

   char szAge[5];
   char szDate[12];

   // set the marital status radio buttons to Single
   // Note that because this is grouped, user taps will automatically 
   // turn off the untapped selections, but programmatically we have to
   // do it ourselves.
   CtlSetValue ( pCtlSingle, 1 );
   CtlSetValue ( pCtlMarried, 0 );
   CtlSetValue ( pCtlDivorced, 0 );

   // set the label for age to 35
   s_Age = 35;
   StrPrintF ( szAge, "%d", s_Age );

   FrmHideObject (formP, wIDAge);
   FrmCopyLabel (formP, MainAgeLabel, szAge);
   FrmShowObject (formP, wIDAge);


   // set the date of birth label on the selector button to be 12/31/63
   Int16 mth, day, year;
   mth = 12;
   day = 31;
   year = 1963;
   StrPrintF ( szDate, "%d/%d/%d", mth, day, year );

	char *pLabel = (char *)CtlGetLabel (pCtlDateofBirth);
	StrCopy (pLabel, szDate);
	CtlSetLabel (pCtlDateofBirth, pLabel);


   // set the checkbox for "contact me" to no
   CtlSetValue ( pCtlContactMe, 0 );


}

/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*formP*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormButtonHandler (FormPtr formP, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      case MainAgeUpRepeating:
      {
         // decrement the age
         if ( s_Age > 22 )
         {
            Int16 wIDAge = FrmGetObjectIndex (formP, MainAgeLabel );
            ControlPtr pCtlAge = (ControlPtr) FrmGetObjectPtr (formP, wIDAge);
            char szAge[5];

            s_Age--;
            StrPrintF ( szAge, "%d", s_Age );
            FrmHideObject (formP, wIDAge);
            FrmCopyLabel (formP, MainAgeLabel, szAge);
            FrmShowObject (formP, wIDAge);
         }
         break;
      }
      case MainAgeDownRepeating:
      {
         // increment the age
         if ( s_Age < 65 )
         {
            Int16 wIDAge = FrmGetObjectIndex (formP, MainAgeLabel );
            ControlPtr pCtlAge = (ControlPtr) FrmGetObjectPtr (formP, wIDAge);
            char szAge[5];

            s_Age++;
            StrPrintF ( szAge, "%d", s_Age );

            FrmHideObject (formP, wIDAge);
            FrmCopyLabel (formP, MainAgeLabel, szAge);
            FrmShowObject (formP, wIDAge);
         }
         break;
      }
      case MainDateofBirthSelTrigger:
      {
         // tapped the selector trigger button control
         // show the selectday dialog, and update the date of birth label upon 
         // return
         Int16 mth, day, year;
         mth = 12;
         day = 31;
         year = 1963;

         if ( SelectDay ( selectDayByDay, &mth, &day, &year, "Date of Birth") )
         {
            ControlPtr pCtlDateofBirth = (ControlPtr)eventP->data.ctlEnter.pControl;
            char szDate[10];

            StrPrintF ( szDate, "%d/%d/%d", mth, day, year );
            CtlSetLabel ( pCtlDateofBirth, szDate );
         }
         handled = true;
         break;
      }

      case MainSubmitButton:   
      {
         char szMessage[100];

         // get the values from the survey form
         Int16 wIDContactMe = FrmGetObjectIndex (formP, MainContactMeCheckbox );
         ControlPtr pCtlContactMe = (ControlPtr) FrmGetObjectPtr (formP, wIDContactMe);

         Int16 wIDAge = FrmGetObjectIndex (formP, MainAgeLabel );
         ControlPtr pCtlAge = (ControlPtr) FrmGetObjectPtr (formP, wIDAge);

         Int16 wIDSingle = FrmGetObjectIndex (formP, MainSinglePushButton );
         ControlPtr pCtlSingle = (ControlPtr) FrmGetObjectPtr (formP, wIDSingle);

         Int16 wIDMarried = FrmGetObjectIndex (formP, MainMarriedPushButton );
         ControlPtr pCtlMarried = (ControlPtr) FrmGetObjectPtr (formP, wIDMarried);

         Int16 wIDDivorced = FrmGetObjectIndex (formP, MainDivorcedPushButton );
         ControlPtr pCtlDivorced = (ControlPtr) FrmGetObjectPtr (formP, wIDDivorced);

         Int16 wIDDateofBirth = FrmGetObjectIndex (formP, MainDateofBirthSelTrigger);
         ControlPtr pCtlDateofBirth = (ControlPtr) FrmGetObjectPtr (formP, wIDDateofBirth);

         const char * pAge = CtlGetLabel ( pCtlAge );
         const char * pDateofBirth = CtlGetLabel ( pCtlDateofBirth );

         char szContactMe[20];
         char szMarital[50];
         char szAge[20];
         char szDateofBirth[50];

         StrPrintF ( szAge, "Age = %s", pAge );
         StrPrintF ( szDateofBirth, "DOB = %s", pDateofBirth );

         if ( CtlGetValue ( pCtlContactMe ) )
         {
            StrCopy ( szContactMe, "Contact me" );
         }
         else
         {
            StrCopy ( szContactMe, "Do not contact me" );
         }
         if ( CtlGetValue ( pCtlSingle ) )
         {
            StrCopy ( szMarital, "Marital Status = Single" );
         }
         else if ( CtlGetValue ( pCtlMarried ) )
         {
            StrCopy ( szMarital, "Marital Status = Married" );
         }
         else 
         {
            StrCopy ( szMarital, "Marital Status = Divorced" );
         }


         StrPrintF ( szMessage, "%s, %s, %s, %s", szMarital, szAge, szDateofBirth, szContactMe );

         PalmMessageBox (MessageBoxAlert, szMessage);

         handled = true;
         break;
      }

      case MainResetButton:   
      {
         ResetForm (formP);
         handled = true;
         break;
      }

   }
   return handled;
}
/*
   PalmMessageBox:

   Toolbox api providing a general purpose message box function

   Parms: wAlertID = id of alert to show
          pMessage = ptr to message text to display

   Returns: id of button chosen by user
*/

Int16
PalmMessageBox ( Int16 wAlertID, char * pMessage )
{
   Int16 err;

   err = FrmCustomAlert (wAlertID, pMessage, "", "" );

   return err;
}

