/*
   MENU_MAI.CPP
   Main form handling functions.
   Copyright (c) 1999-2002 Bachmann Software and Services
   Author: Glenn Bachmann
*/

// system headers
#include <PalmOS.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "menus.h"      // common app header
#include "menu_res.h"   // resource ids

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);

Int16 PalmMessageBox ( Int16 wAlertID, char * pMessage );

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      
      case ctlSelectEvent:
      {
         // A control button was tapped and released
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         // main form is opening

         FormPtr formP = FrmGetActiveForm ();

         MainFormInit (formP);

         FrmDrawForm (formP);

         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr formP)
{
   // set the controls to an initial state
}

/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*formP*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      case RecordNew:
      {
         PalmMessageBox (MessageBoxAlert, "New Record");
         break;
      }
      case RecordDelete:
      {
         PalmMessageBox (MessageBoxAlert, "Delete Record");
         break;
      }
      case RecordPurge:
      {
         PalmMessageBox (MessageBoxAlert, "Purge");
         break;
      }
      case EditUndo:
      {
         // get the currently active field object, and call the FldUndo
         // api
         PalmMessageBox (MessageBoxAlert, "Undo");
         break;
      }
      case EditCut:
      {
         // get the currently active field object, and call the FldCut
         // api
         PalmMessageBox (MessageBoxAlert, "Cut");
         break;
      }
      case EditCopy:
      {
         // get the currently active field object, and call the FldCopy
         // api
         PalmMessageBox (MessageBoxAlert, "Copy");
         break;
      }
      case EditPaste:
      {
         // get the currently active field object, and call the FldPaste
         // api
         PalmMessageBox (MessageBoxAlert, "Paste");
         break;
      }
      case EditKeyboard:
      {
         // invoke the keyboard dialog, with the default keyboard
         // NOTE: this only works if there is a field object with the focus!
         // SysKeyboardDialog (kbdDefault);
         PalmMessageBox (MessageBoxAlert, "Keyboard");
         break;
      }
      case EditGraffitiHelp:
      {
         // invoke the graffiti ref dialog, with the default reference
         SysGraffitiReferenceDialog (referenceDefault);
         break;
      }
      case OptionsAboutMenus:
      {
         // invoke the graffiti ref dialog, with the default reference
         PalmMessageBox (MessageBoxAlert, "About");
         break;
      }
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormButtonHandler (FormPtr formP, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   PalmMessageBox:

   Toolbox api providing a general purpose message box function

   Parms: wAlertID = id of alert to show
          pMessage = ptr to message text to display

   Returns: id of button chosen by user
*/

Int16
PalmMessageBox ( Int16 wAlertID, char * pMessage )
{
   Int16 err;

   err = FrmCustomAlert (wAlertID, pMessage, "", "" );

   return err;
}
