/*
   DRAW_MAI.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1999-2002
   Author: Glenn Bachmann
*/

// system headers
#include <PalmOS.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "draw.h"
#include "draw_res.h"

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);

void  DrawGameBoard ( void );

static UInt16 s_x = 0;
static UInt16 s_y = 0;

static Boolean s_bPenDown = false;

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      
      case penDownEvent:
      {
         if ( eventP->screenY < 130 )
         {
            s_x = eventP->screenX;
            s_y = eventP->screenY;

            s_bPenDown = true;
            return true;
         }
         else 
         {
            return false;
         }
      }

      case penUpEvent:
      {
         if ( s_bPenDown && eventP->screenY < 130 )
         {
            UInt16 xNew = eventP->screenX;
            UInt16 yNew = eventP->screenY;

            WinDrawLine ( s_x, s_y, xNew, yNew );

            s_x = 0;
            s_y = 0;

            s_bPenDown = false;
         }

         break;
      }

      case penMoveEvent:
      {
         // only do something if the pen is down
         if ( s_bPenDown && eventP->screenY < 130 )
         {
            UInt16 xNew = eventP->screenX;
            UInt16 yNew = eventP->screenY;

            WinDrawLine ( s_x, s_y, xNew, yNew );

            s_x = xNew;
            s_y = yNew;
         }
         break;
      }

      case ctlSelectEvent:
      {
         // A control button was tapped and released
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         // main form is opening

         FormPtr formP = FrmGetActiveForm ();

         MainFormInit (formP);

         FrmDrawForm (formP);

         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr formP)
{
   // set the controls to an initial state

   DrawGameBoard ();

}

/*
   DrawGameBoard:
   Redraws the tic tac toe display
   Parms: none
   Returns: none
*/
void 
DrawGameBoard ( void )
{
   // our game area is 120 pixels high,  starts after the title area,
   // and ends above the button area on the bottom
   WinDrawLine ( 53, 16, 53, 136 );
   WinDrawLine ( 106, 16, 106, 136 );

   // horizontal lines
   WinDrawLine ( 0, 56, 159, 56 );
   WinDrawLine ( 0, 96, 159, 96 );
   return;
}

/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*formP*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormButtonHandler (FormPtr formP, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      case MainEraseButton:
      {
         
         RectangleType rect;
         rect.topLeft.x = 0;
         rect.topLeft.y = 16;
         rect.extent.x = 159;
         rect.extent.y = 136 - rect.topLeft.y;

         WinEraseRectangle ( &rect, 0);
         handled = true;
         break;
      }
      default: 
      {
         handled = false;
         break;
      }
   }
   return handled;
}

