/*
   PLM_MAIN.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1999-2002
   Author: Glenn Bachmann
*/

// system headers
#include <PalmOS.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "plm_main.h"
#include "plm_res.h"

static void    MainFormInit (FormPtr formP);
static void 	MainFormClose( FormPtr pForm );
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);


/*
   MainFormEventHandler (EventPtr)
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr pEvent)
{
   Boolean  handled = false;
   Int32 lError = 0;
   FormPtr pForm = FrmGetActiveForm ();

   switch (pEvent->eType)
   {
      case menuEvent:
      {
         handled = MainFormMenuHandler (pForm, pEvent);
         break;
      }
      case ctlSelectEvent:
      {
         handled = MainFormButtonHandler ( pForm, pEvent );
         break;
      }
      case frmCloseEvent:
      {
         MainFormClose ( pForm );
         break;
      }
      case frmOpenEvent:
      {
         // main form is opening
         MainFormInit (pForm);
         FrmDrawForm ( pForm );
         handled = true;
         break;
      }
      default:
      {
         break;
      }
   }

   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   pForm - pointer to main form.
   Return:  none
*/
void
MainFormInit ( FormPtr pForm )
{
	
}


void
MainFormClose ( FormPtr pForm )
{
}


/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   pForm    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*pForm*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {

      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}


/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   pForm    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/

Boolean
MainFormButtonHandler (FormPtr pForm, EventPtr eventP)
{
   Boolean handled = false;
   SystemPreferencesType prefType;

   switch (eventP->data.ctlEnter.controlID)
   {
      case MainPickColorButton:
      {
			Boolean b;
			IndexedColorType index;
			RGBColorType rgb;
			
			Err err;
			UInt32 uiRomVersion;
			UInt32 uiVersion35 = sysMakeROMVersion(3,5,0,sysROMStageRelease,0);
 
			err = FtrGet(sysFtrCreator,
								sysFtrNumROMVersion, &uiRomVersion);
					
			index = 0;
			rgb.r = 0;
			rgb.g = 0;
			rgb.b = 0;
			
			b = UIPickColor (&index, 
								  &rgb, 
								UIPickColorStartPalette,
								"Pick a Color, Any Color", 
								NULL);
			if (b)
			{
				char szRGB[40];
				IndexedColorType oldIndex, newIndex;
				RectangleType rect;
				
				StrPrintF ( szRGB, "%d red %d green %d blue", rgb.r, rgb.g, rgb.b );
				
				FrmCustomAlert (ErrorAlert, szRGB, NULL, NULL );
				
				newIndex = WinRGBToIndex ( &rgb );
				oldIndex = WinSetForeColor ( newIndex );
			  	RctSetRectangle ( &rect, 0, 17, 159, 14);
				WinDrawRectangle (&rect, 0);
				WinSetForeColor (oldIndex );
			
			}								
         break;
      }
      
      case MainFormFillButton:
      {
			Boolean b;
			RGBColorType rgb;
			Err err;
			
			IndexedColorType oldIndex, newIndex;
			RectangleType rect;
			
			UIColorGetTableEntryRGB ( UIFormFill, &rgb );
			
			b = UIPickColor (NULL, 
								  &rgb, 
								UIPickColorStartRGB,
								"Pick a Color, Any Color", 
								NULL);
			if (b)
			{
				newIndex = WinRGBToIndex ( &rgb );
				oldIndex = WinSetForeColor ( newIndex );
			  	RctSetRectangle ( &rect, 80, 31, 80, 14);
				WinDrawRectangle (&rect, 0);
				WinSetForeColor (oldIndex );
				
				UIColorSetTableEntry ( UIFormFill, &rgb );
				FrmDrawForm (pForm);	
      	}
      	break;
      }
      default: 
      {
         handled = false;
         break;
      }
   }
   return handled;
}

