/*
   DBB_MAIN.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1999-2002
   Author: Glenn Bachmann
*/

// system headers
#include <PalmOS.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "dbbrowse.h"
#include "dbb_res.h"

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);


Err PalmMessageBox ( Int16 wAlertID, char * pMessage );

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      

      case ctlSelectEvent:
      {
         // A control button was tapped and released
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         // main form is opening

         FormPtr formP = FrmGetActiveForm ();

         MainFormInit (formP);

         FrmDrawForm (formP);

         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr formP)
{
   // set the controls to an initial state

}

/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*formP*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormButtonHandler (FormPtr formP, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      case MainOKButton:
      {
         Err err;
         DmSearchStateType state;
         UInt16 uCardNo;
         LocalID dbID;

         err = DmGetNextDatabaseByTypeCreator (
                                                true, // new search
                                                &state,
                                                0,    // all types
                                                0,    // all creators
                                                true, // only latest vers
                                                &uCardNo, // card number
                                                &dbID );
         while ( 0 == err )
         {
            
            char szName[33];
            char szMessage[100];
            UInt32 ulNumRecords;
            UInt32 ulBytes;

            DmDatabaseInfo ( uCardNo,
                             dbID,
                             szName,
                             0,
                             0,
                             0,
                             0,
                             0,
                             0,
                             0,
                             0,
                             0,
                             0 );


            DmDatabaseSize ( uCardNo,
                              dbID,
                              &ulNumRecords,
                              &ulBytes,
                              0 );

            StrPrintF ( szMessage, "%s %lu %lu", szName, ulNumRecords, ulBytes );

            PalmMessageBox ( MessageBoxAlert, szMessage );


            err = DmGetNextDatabaseByTypeCreator (
                                                   false, // not a new search
                                                   &state,
                                                   0,    // all types
                                                   0,    // all creators
                                                   true, // only latest vers
                                                   &uCardNo, // card number
                                                   &dbID );
         }
                                                
         handled = true;
         break;
      }
      default: 
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   PalmMessageBox:

   Toolbox api providing a general purpose message box function

   Parms: wAlertID = id of alert to show
          pMessage = ptr to message text to display

   Returns: id of button chosen by user
*/

Err
PalmMessageBox ( Int16 wAlertID, char * pMessage )
{
   Err err;

   err = FrmCustomAlert (wAlertID, pMessage, "", "" );

   return err;
}


