/*
   CATE_MAI.CPP
   Main form handling functions.
   Copyright (c) Bachmann Software and Services, 1999
   Author: Glenn Bachmann
*/

// system headers
#include <Pilot.h>
#include <SysEvtMgr.h>

// application-specific headers
#include "category.h"
#include "cate_res.h"

static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);

// local helper functions
void MainListFill (FormPtr pForm);
void MainListDrawFunction (UInt itemNum, RectanglePtr bounds, CharPtr *pUnused);
Boolean AddFish ( void );
Boolean AddFish_EventHandler (EventPtr pEvent);

/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      

      case ctlSelectEvent:
      {
         // A control button was tapped and released
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         // main form is opening

         FormPtr formP = FrmGetActiveForm ();

         MainFormInit (formP);

         FrmDrawForm (formP);

         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr pForm)
{
   char szType [dmCategoryLength];

   // set the controls to an initial state
   // here we set the name of the currently selected category
   FishGetCurrentType ( szType );

   Word wID = FrmGetObjectIndex (pForm, MainTypePopTrigger );
   ControlPtr p = (ControlPtr) FrmGetObjectPtr (pForm, wID);

	CategorySetTriggerLabel ( p, szType );

   MainListFill ( pForm );

   // set the drawing function for the main list
   Word wIDMainList = FrmGetObjectIndex (pForm, MainFishList );
   ListPtr pCtlMainList = (ListPtr) FrmGetObjectPtr (pForm, wIDMainList);
   LstSetDrawFunction ( pCtlMainList, MainListDrawFunction );
}

/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*formP*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      default:
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/

Boolean
MainFormButtonHandler (FormPtr pForm, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      case MainNewButton:
      {
         if ( AddFish () )
         {
            MainListFill ( pForm );
         }

         handled = true;
         break;
      }

      case MainTypePopTrigger:
      {  
         FormPtr pForm;
         UInt wType;

      	pForm = FrmGetActiveForm ();

         if ( FishSelectType ( pForm, &wType, MainTypePopTrigger, MainTypeList ) )
         {
		      MainListFill ( pForm );
         }
         handled = true;
         break;
      }
      default: 
      {
         handled = false;
         break;
      }
   }
   return handled;
}

/*
   MainListFill:
   Fills the fish list based upon the currently selected category
   Parms: pForm - pointer to main form
   Returns: none
*/
void
MainListFill (FormPtr pForm)
{
   // get a ptr to the fish list
   Word wIDFishList = FrmGetObjectIndex (pForm, MainFishList );
   ListPtr pCtlFishList = (ListPtr) FrmGetObjectPtr (pForm, wIDFishList);

   // get the count of available main list items from the selected aisle
   int iFishCount = FishGetCount ();

   // fill the main list based on the combo selection
   LstSetListChoices ( pCtlFishList, NULL, iFishCount );

   // redraw the list
   LstDrawList (pCtlFishList);

   return;
}

/*
   MainListDrawFunction:
   Draws a single item in the main list
   Parms: itemNum = 0 based index of item to draw
          bound = ptr to rectangle providing drawing bounds
          pUnused = ptr to char string for item to draw. In this app, we
          never give the actual item data to the list, so we ignore this parm
   Returns: none
*/
void 
MainListDrawFunction (UInt itemNum, RectanglePtr bounds, CharPtr *pUnused)
{
   FormPtr pForm = FrmGetActiveForm ();

	CharPtr		pText;
   Handle      hFish = NULL;
	
   // find the record in the category
   FishGetFish ( itemNum );

   pText = FishGetName ();

	if ( pText )
   {
   	WinDrawChars ( pText, StrLen (pText), bounds->topLeft.x, bounds->topLeft.y);
   }
	
   FishReleaseFish ();
   return;
}


/*
   AddFish:
   Parms:   none
   Return:  none
*/
Boolean AddFish ( void )
{
   // initialize the form
   FormPtr  pForm = FrmInitForm (AddFishForm);

	if ( FishNew () )
   {
      // edit it!
      VoidHand hRecord = FishGetFish ( 0 );
      VoidHand hText = MemHandleNew ( FISH_LENGTH );
      CharPtr pText = (CharPtr)MemHandleLock (hText);

      StrCopy ( pText, "<new fish>" );

   	MemPtrUnlock ( pText );


      Word wIDField = FrmGetObjectIndex ( pForm, AddFishNameField );
      FieldPtr pField = (FieldPtr) FrmGetObjectPtr ( pForm, wIDField);

      // copy the text into the edit field
		FldSetTextHandle ( pField, (Handle)hText);

      // set the current trigger label
      char szType [dmCategoryLength];
      FishGetCurrentType ( szType );
      Word wID = FrmGetObjectIndex (pForm, AddFishTypePopTrigger );
      ControlPtr p = (ControlPtr) FrmGetObjectPtr (pForm, wID);
	   CategorySetTriggerLabel ( p, szType );

	   // we set an event handler for the form
	   FrmSetEventHandler (pForm, AddFish_EventHandler );

      // display the dialog
      if ( FrmDoDialog (pForm) )
      {
         hText = FldGetTextHandle (pField);
         CharPtr pText = (CharPtr)MemHandleLock (hText);

         FishSetName ( pText );

      	MemPtrUnlock ( pText );
      }

      FishReleaseFish ();

      // destroy the form
      FrmDeleteForm (pForm);

      return true;
   }
   return false;
}


/*
   AddFish_EventHandler:
	Handles form events for form
   Parms:   pEvent = ptr to event handler
   Return:  TRUE if we handled the event
*/

Boolean AddFish_EventHandler (EventPtr pEvent)
{
   Boolean  bHandled = false;
   static UInt s_wNewType;

   switch (pEvent->eType)
   {
      case ctlSelectEvent:
		{
      	// A control button was pressed and released.	
		   switch (pEvent->data.ctlSelect.controlID)
			{
			   case AddFishOKButton:
            {
               // save the fish!
               FishUpdate ();

		   		// Remove the details form and display the edit form.
   				bHandled = false;
	   			break;
            }

			   case AddFishCancelButton:
            {
	   		   // The cancel button was pressed, just go back to the edit form.
				   bHandled = false;
				   break;
            }
				
   			case AddFishTypePopTrigger:
            {
               // user wants to pick a fishtype
               FormPtr pForm = FrmGetActiveForm ();
               FishSelectType ( pForm, &s_wNewType, AddFishTypePopTrigger, AddFishTypeList );
               bHandled = true;
               break;
            }
			}
		}
	}
   return bHandled;
}


