/******************************************************************************
 * File: EMSample.c
 * Author: W.F. Ableson
 * Description: Example application for Exchange Manager Programming
 *****************************************************************************/

#include <PalmOS.h>
#include <ExgMgr.h>
#include "EMSampleRsc.h"


/***********************************************************************
 *
 *   Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/
typedef struct 
	{
	UInt8 replaceme;
	} EMSamplePreferenceType;

typedef struct 
	{
	UInt8 replaceme;
	} EMSampleAppInfoType;

typedef EMSampleAppInfoType* EMSampleAppInfoPtr;


/***********************************************************************
 *
 *   Global variables
 *
 ***********************************************************************/
#define MAXPOSITIONS                5
#define MAXSIZE                     (2 * 1024)
#define FIELDSIZE                   50
#define EXTENSION                   ".arbitrary"
#define REGISTRATIONEXTENSION       "arbitrary"

Char lasterr[200];
Char textchoices[MAXPOSITIONS][FIELDSIZE];
Char *textpointers[MAXPOSITIONS];
UInt8 loopbackmode = 0;


/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
#define appFileCreator				 'EMSP'
#define appVersionNum              0x01
#define appPrefID                  0x00
#define appPrefVersionNum          0x01

// Define the minimum OS version we support (3.0 for now).
#define ourMinVersion	sysMakeROMVersion(3,0,0,sysROMStageRelease,0)


/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID);
void MsgBox(Char * s);
void ErrBox(Char * s);
UInt16 ConfirmBox(Char *s);
void FillBox(ListPtr listP);
LocalID FindDB();
UInt16 RemoveData(Char *name);
UInt16 SendData(Char *name);
UInt16 StoreData(Char * name,Char * data,UInt16 size);




void MsgBox(Char * s)
{
   FrmCustomAlert(InfoAlert,s,NULL,NULL);
}


void ErrBox(Char * s)
{
   FrmCustomAlert(ErrorAlert,s,NULL,NULL);
}

UInt16 ConfirmBox(Char *s)
{
   return FrmCustomAlert(ConfirmAlert,s,NULL,NULL);
}


LocalID FindDB()
{
LocalID dbid;
Err err;
	dbid = DmFindDatabase(0,"ArbitraryDB");
	if (dbid == 0)
	{
		// create db
		err = DmCreateDatabase(0,"ArbitraryDB",appFileCreator,'data',false);
		if (err != 0)
		{
			// failed to create database
			StrPrintF(lasterr,"Failed to Create Arbitrary Database");
			ErrBox(lasterr);
			return 0;
		}
		dbid = DmFindDatabase(0,"ArbitraryDB");
		if (dbid == 0)
		{
			StrPrintF(lasterr,"Failed to Find Arbitrary Database");
		   ErrBox(lasterr);
			return 0;
		}
	}
	return dbid;
}
      
void FillBox(ListPtr listP)
{
DmOpenRef dbref;
UInt16 recindex = 0;
UInt16 reccount = 0;
UInt16 i;
MemHandle memH;
Char * memP;
Char * p2;
EventType evt;   
LocalID dbid;
   
   // initialize list pointers
   for (i=0;i<MAXPOSITIONS;i++)
   {
      textpointers[i] = NULL;
   }
   
   dbref = 0;
   dbid = FindDB();
	dbref = DmOpenDatabase(0,dbid,dmModeReadWrite);
	if (dbref == 0)
	{
		// failed to open database?
		StrPrintF(lasterr,"Failed to Open Arbitrary Database");		
		ErrBox(lasterr);
		goto leave;
	}		
	// copy out the textual names of the records ....

	reccount = DmNumRecords(dbref);
   if (reccount == 0)
   {
      DmCloseDatabase(dbref);
      MsgBox("There are no arbitrary records present.\nCreating sample record.");
      if (0 == StoreData("sample.arbitrary","blah",4))
      {
         dbref = DmOpenDatabase(0,dbid,dmModeReadWrite);
         if (dbref == 0)
         {
      		// failed to re-open database?
      		StrPrintF(lasterr,"Failed to Re-Open Arbitrary Database");		
      		ErrBox(lasterr);
      		goto leave;
      	}		
      }
   }

   reccount = DmNumRecords(dbref);
	for (i=0;i<reccount;i++)
	{
	   if (i >= MAXPOSITIONS)
	   {
	      i = reccount+1;
	      continue;
	   }
		memH = DmQueryRecord(dbref,i);
		if (memH == 0)
		{
			StrPrintF(lasterr,"failed to get record %d",DmGetLastErr());
			ErrBox(lasterr);
			goto leave;
		}
		// lock down this record
		memP = (Char *) MemHandleLock(memH);
		if (!memP)
		{
			// uhoh, this should not happen!
			StrPrintF(lasterr,"failed to lock down record %d",DmGetLastErr());
			ErrBox(lasterr);
	   }
	   else
	   {
	      MemSet((Char *)&textchoices[i][0],FIELDSIZE,0x00);
	      p2 = (Char *) StrStr(memP,"|");
	      if (p2 == NULL)
	      {
	         StrCopy((Char *) &textchoices[i][0],"Unavailable?");
	      }
	      else
	      {
	         StrNCopy((Char *) &textchoices[i][0],memP,p2-memP);
	      }
	      textpointers[i] = &textchoices[i][0];
	      MemHandleUnlock(memH);
	   }
	}
	
   DmCloseDatabase(dbref);
   LstSetListChoices(listP,textpointers,reccount);   
   if (reccount == 1)
   {
      LstSetHeight(listP,1);
   }
   else
   {
      LstSetHeight(listP,2);
   }
   LstSetSelection(listP,0);
   MemSet(&evt,sizeof(evt),0x00);
   evt.eType = popSelectEvent;
   evt.data.popSelect.listID = MainContentList;
   evt.data.popSelect.listP = listP;
   evt.data.popSelect.selection = 0;
   evt.data.popSelect.controlID = MainContentPopTrigger;
   evt.data.popSelect.controlP = (ControlPtr) GetObjectPtr(MainContentPopTrigger);
   evt.data.popSelect.priorSelection = -1;
   EvtAddEventToQueue(&evt);
   
   
leave:
   return;   
}         


/*
SendData
Params: name of record to send 
*/
UInt16 SendData(Char *name)
{
ExgSocketType sock;
Char namebuf[50];
Char buf[100];
UInt32 totalbytes,bytes,bytessent;
   
UInt16 ret = (UInt16) -1;   
Char * p;
DmOpenRef dbref;
UInt16 recindex = 0;
UInt16 reccount = 0;
MemHandle memH;
Char * memP;
UInt16 i;
LocalID dbid;
Err err;
   
   dbid = FindDB();
   // open database
	dbref = DmOpenDatabase(0,dbid,dmModeReadOnly);
	if (dbref == 0)
	{
		// failed to open database?
		StrPrintF(lasterr,"Failed to Open Arbitrary Database for SendData");		
		ErrBox(lasterr);
      return ret;
	}	

   reccount = DmNumRecords(dbref);
	for (i=0;i<reccount;i++)
	{
	   if (i >= MAXPOSITIONS)
	   {
	      i = reccount+1;
	      continue;
	   }
		memH = DmQueryRecord(dbref,i);
		if (memH == 0)
		{
			StrPrintF(lasterr,"failed to get record %d",DmGetLastErr());
			ErrBox(lasterr);
			goto leave;
		}
		// lock down this record
		memP = (Char *) MemHandleLock(memH);
		if (!memP)
		{
			// uhoh, this should not happen!
			StrPrintF(lasterr,"failed to lock down record %d",DmGetLastErr());
			ErrBox(lasterr);
			goto leave;
	   }
	   else
	   {
         // compare names ...
         
	      MemSet(buf,sizeof(buf),0x00);
	      p = (Char *) StrStr(memP,"|");
	      if (p == NULL)
	      {
	         // hmmm.. bad record ....!
   	      MemHandleUnlock(memH);
	      }
	      else
	      {
	         StrNCopy(buf,memP,p-memP);
	         if (!StrCompare(buf,name))
	         {
	            // this is the record to send!
	            MemSet(&sock,sizeof(sock),0x00);
	            StrPrintF(namebuf,"%s%s",name,EXTENSION);
               //MsgBox(namebuf);
               // calculate how much data is to be sent ...
               totalbytes = MemHandleSize(memH);
               // we want to send the total record,less the size of the name and the delimiter
               bytes = totalbytes - StrLen(name) - 1;

  	            sock.name = namebuf;
	            sock.description = namebuf;
	            sock.length = bytes;
	            sock.count = 1;
	            sock.localMode = loopbackmode;
	            err = ExgPut(&sock);
	            if (err)
	            {
	               StrPrintF(buf,"Failed to initiate OBEX Put.  Error [%d]",err);
	               ErrBox(buf);
           	      MemHandleUnlock(memH);
           	      ExgDisconnect(&sock,err);
           	      goto leave;
               }                                 	              
               else
               {
                  // let's send this data across ....  
                  // skip past delimiter
                  p++;
                  //StrPrintF(buf,"bytes =  %ld",bytes);
                  //MsgBox(buf);
                  err = 0;
                  while (err == 0 && bytes > 0)
                  {
                     bytessent = ExgSend(&sock,p,bytes,&err);
                     bytes -= bytessent;
                     p += bytessent;
                  }
                  if (err || bytes > 0)
                  {
                     ErrBox("Error, data not completely exchanged!");
                  }
                  ExgDisconnect(&sock,err);  
               }
        	      MemHandleUnlock(memH);
        	      // jump out of loop cleanly
        	      i = reccount+1; 
            }	                   
	      }
	   }
	}
	
   
   ret = 0;
leave:     
   DmCloseDatabase(dbref);
   return ret; 

}   

/*
RemoveData
Parameters: name of "file"
*/
UInt16 RemoveData(Char *name)
{
UInt16 ret = (UInt16) -1;   
Char * p;
DmOpenRef dbref;
UInt16 recindex = 0;
UInt16 reccount = 0;
MemHandle memH;
Char * memP;
Char buf[100];
UInt16 i;
LocalID dbid;
   
   dbid = FindDB();
   // open database
	dbref = DmOpenDatabase(0,dbid,dmModeReadWrite);
	if (dbref == 0)
	{
		// failed to open database?
		StrPrintF(lasterr,"Failed to Open Arbitrary Database for StoreData");		
		ErrBox(lasterr);
      return ret;
	}	

   reccount = DmNumRecords(dbref);
	for (i=0;i<reccount;i++)
	{
	   if (i >= MAXPOSITIONS)
	   {
	      i = reccount+1;
	      continue;
	   }
		memH = DmQueryRecord(dbref,i);
		if (memH == 0)
		{
			StrPrintF(lasterr,"failed to get record %d",DmGetLastErr());
			ErrBox(lasterr);
			goto leave;
		}
		// lock down this record
		memP = (Char *) MemHandleLock(memH);
		if (!memP)
		{
			// uhoh, this should not happen!
			StrPrintF(lasterr,"failed to lock down record %d",DmGetLastErr());
			ErrBox(lasterr);
			goto leave;
	   }
	   else
	   {
         // compare names ...
         
	      MemSet(buf,sizeof(buf),0x00);
	      p = (Char *) StrStr(memP,"|");
	      if (p == NULL)
	      {
	         // hmmm.. bad record ....!
   	      MemHandleUnlock(memH);
	      }
	      else
	      {
	         StrNCopy(buf,memP,p-memP);
	         if (!StrCompare(buf,name))
	         {
	            //MsgBox("Match Found");
        	      MemHandleUnlock(memH);
        	      DmRemoveRecord(dbref,i);
        	      // jump out of loop cleanly
        	      i = reccount+1; 
            }	                   
	      }
	   }
	}
	ret = 0;
leave:     
   DmCloseDatabase(dbref);
   return ret; 
}

/*
StoreData
Parameters: name of "file"
            data to store
            size of payload
*/
UInt16 StoreData(Char * name,Char * data,UInt16 size)
{
UInt16 ret = (UInt16) -1;   
UInt16 totallength;
Char * p;
DmOpenRef dbref;
UInt16 recindex = 0;
UInt16 reccount = 0;
MemHandle memH;
Char * memP;
Char buf[100];
LocalID dbid;


   // sanity check on the size
   if (size > MAXSIZE)
   {
      ErrBox("Data is too large to store!");
      return ret;
   }
   
   dbid = FindDB();
   // open database
	dbref = DmOpenDatabase(0,dbid,dmModeReadWrite);
	if (dbref == 0)
	{
		// failed to open database?
		StrPrintF(lasterr,"Failed to Open Arbitrary Database for StoreData");		
		ErrBox(lasterr);
      return ret;
	}	
	reccount = DmNumRecords(dbref);
   if (reccount >= MAXPOSITIONS)
   {
      // there are already too many records here 
      ErrBox("You must remove a record before storing this new data");
      goto leave;
   }
   
  
   // let's add this data
   MemSet(buf,sizeof(buf),0x00);
   // pull off the extension
   p = (Char *) StrStr(name,EXTENSION);
   if (p)
   {
      StrNCopy(buf,name,p-name);
   }
   totallength = StrLen(buf) + 1 + size + 1;   
   StrCat(buf,"|");
   recindex = dmMaxRecordIndex;
   // let's create a single default record   
	memH = DmNewRecord(dbref,&recindex,totallength);
	if (memH == 0)
	{
		// uhoh
		ErrBox("Failed to Create New Record");
		goto leave;
	}
	memP = (Char *) MemHandleLock(memH);
	if (memP == 0)
	{
		// uhoh
		ErrBox("Failed to lock newly created record");
		DmReleaseRecord(dbref,recindex,false);
		goto leave;
	}
	DmWrite(memP,0,buf,StrLen(buf));
	DmWrite(memP,StrLen(buf),data,size);
	MemHandleUnlock(memH);
	DmReleaseRecord(dbref,recindex,true);
   ret = 0; 		
leave:   
   if (dbref != 0) DmCloseDatabase(dbref);
   return ret;
}   

/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < ourMinVersion)
				{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return sysErrRomIncompatible;
		}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}


/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr /*frmP*/)
{
   FillBox(GetObjectPtr(MainContentList));   
}


/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
   FormPtr frmP;
   Err err;
   
	switch (command)
		{
		case MainOptionsAboutEMSample:
			MenuEraseStatus(0);					// Clear the menu status from the display.
			frmP = FrmInitForm (AboutForm);
			FrmDoDialog (frmP);					// Display the About Box.
			FrmDeleteForm (frmP);
			handled = true;
			break;

	   case MainOptionsRegisterwithExchangeMgr:
         // let's register for the arbitrary data type!
         err = ExgRegisterData(appFileCreator,exgRegExtensionID,REGISTRATIONEXTENSION);

	      handled = true;
	      break;
	   case MainOptionsUnregisterwithExchangeMgr:
         // let's un register this application
         err = ExgRegisterData(appFileCreator,exgRegExtensionID,NULL);

	      handled = true;
	      break;
	   case MainOptionsSetLoopbackMode:
	      loopbackmode = 1;
	      handled = true;
	      break;
	   case MainOptionsUnsetLoopbackMode:
	      loopbackmode = 0;
	      handled = true;
	      break;
	      
	   }
	   
	return handled;
}







static Boolean MainFormHandleButton(FormPtr formP,EventPtr eventP)
{
   Boolean handled = false;
   UInt16 index = FrmGetObjectIndex (formP, MainContentList);
   ListPtr listP = (ListPtr) FrmGetObjectPtr (formP, index);
   Int16 selectIndex = LstGetSelection (listP);
   Char buf[100];
      
   switch (eventP->data.ctlEnter.controlID)
   {
      
      case MainRemoveButton:
      {
         StrPrintF(buf,"Remove %s.  Are you sure",LstGetSelectionText(listP,selectIndex));
         if (ConfirmBox(buf) == ConfirmYes)
         {
            RemoveData(LstGetSelectionText(listP,selectIndex));
            FillBox(listP);
         }
         handled = true;
         break;
      }
      case MainSendButton:
      {

         SendData(LstGetSelectionText(listP,selectIndex));
         if (loopbackmode) FillBox(listP);
         handled = true;
         break;
      }
   }
   return handled;
}







/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr frmP;

	switch (eventP->eType) 
		{
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			handled = true;
			break;

      case ctlSelectEvent:
         frmP = FrmGetActiveForm ();
         return MainFormHandleButton(frmP,eventP);
		default:
			break;
		
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
EMSamplePreferenceType prefs;
UInt16 prefsSize;
	// Read the saved preferences / saved-state information.
	prefsSize = sizeof(EMSamplePreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
		noPreferenceFound)
		{
		}
	
	
   return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
   EMSamplePreferenceType prefs;

	// Write the saved preferences / saved-state information.  This data 
	// will be backed up during a HotSync.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
		
	// Close all the open forms.
	FrmCloseAllForms ();
}


/***********************************************************************
 *
 * FUNCTION:    EMSamplePalmMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static UInt32 EMSamplePalmMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
Err error;
MemHandle bufH;
MemPtr bufP;
ExgAskParamType * p;
ExgSocketPtr s;	
UInt32 bytes = 0;
Char buf[100];
   
	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
		case sysAppLaunchCmdGoTo:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;
      
      case sysAppLaunchCmdExgAskUser:
         p = (ExgAskParamType *) cmdPBP;
         //StrPrintF(buf,"Receiving [%s]",p->socketP->name);
         //MsgBox(buf);
			//p->result = exgAskOk;
			p->result = exgAskDialog;
      	break;
      case sysAppLaunchCmdExgReceiveData:
         s = (ExgSocketPtr) cmdPBP;
         switch (ExgAccept(s))
         {
         	case errNone:
            	// begin to receive data
               if (s->name) StrPrintF(buf,"name is [%s]",s->name);
               //MsgBox(buf);
               bufH = MemHandleNew(MAXSIZE);
               if (bufH)
               {
                  bufP = MemHandleLock(bufH);
                  if (bufP)
                  {
                     MemSet(bufP,MAXSIZE,0x00);
                     bytes = ExgReceive(s,bufP,MAXSIZE - 1,&error);
                     if (bytes)
                     {
                        StoreData(s->name,bufP,bytes);
                     	//MsgBox(bufP);
                     }                                                
                     else
                     {
                        StrPrintF(buf,"Failed to receive data!Error [%d]",error);
                        ErrBox(buf);
                     }           
                     MemHandleUnlock(bufH);
                     MemHandleFree(bufH);
                     if (loopbackmode == 0) s->goToCreator = appFileCreator;
                     ExgDisconnect(s,0);
                  }
                  else
                  {
                     ErrBox("Failed to lock memory block");
                     ExgDisconnect(s,2);
                  }            
               }
               else
               {
                  ErrBox("Failed to allocate memory block");
                  ExgDisconnect(s,1);
               }
            	break;
            case exgErrBadLibrary :
					ErrBox("Bad Library on accept?");
            	break;
            case exgErrStackInit:
            	ErrBox("Stack Init errors");
               break;
            default:
            	ErrBox("Unrecognized return from ExgAccept");
            	break;
         }
         break;
			
			
			
		default:
			break;

		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return EMSamplePalmMain(cmd, cmdPBP, launchFlags);
}

