/*
   IrDemomain.cpp
   Ir Library Example Program, Chapter 20
   Copyright (c) Bachmann Software and Services, 1999
   Author: W.F. Ableson
*/

#include <Palmos.h>
#include <palmcompatibility.h>
#include <unix_stdarg.h>
#include <SysEvtMgr.h>
#include <irlib.h>
#include "IrDemomain.h"		// MainFormEventHandler ()
#include "IrDemo_res.h"
#include "callback.h"

// callback functions needed by Ir Library
static void IrHandler(IrConnect * pCon,IrCallBackParms * pCBParms);
static void IASHandler(IrStatus irStat);
// end call back functions

// helper functions
static void DisplayRxBuff(IrCallBackParms * pCBParms );
static void OnDiscovery(IrCallBackParms * pCBParms);
static void OnLapConfirmed();
static void OnLMPRequest();
void PalmOS_printf(char * pszFormat, ...);
// end helper functions

// form handling functions
static void    MainFormInit (FormPtr formP);
static Boolean MainFormButtonHandler (FormPtr formP, EventPtr eventP);
static Boolean MainFormMenuHandler (FormPtr formP, EventPtr eventP);
void MainFormStart(FormPtr formP);
void MainFormSend(FormPtr formP);
void MainFormConnect (FormPtr formP);
static void MainFormVersion (FormPtr formP);
// end form handling functions

static long iMediaBusy=0;
static IrConnect IrCon;
static UInt  refNum;
static IrDeviceAddr g_irDevice;
char szDeviceName[128];
static int IrInitializedYet = 0;
static IrPacket irPack = {
	{ 0, 0, },
	0,
	0,
};
static IrIasQuery 	clientQuery;
static Byte				queryResult[16];
static Byte OurDeviceName[] = { IAS_ATTRIB_USER_STRING,IR_CHAR_ASCII,8,'P','A','L','M','D','E','M','O'};
static Byte OurDeviceNameLen = sizeof(OurDeviceName);
static Byte irGetQuery[] = {6,'D','e','v','i','c','e',10,'D','e','v','i','c','e','N','a','m','e'};
static Byte irGetQuerySize = sizeof(irGetQuery);   
static Byte OurDeviceInfo[] = {IR_HINT_PDA, IR_CHAR_ASCII,'P','A','L','M','D','E','M','O'};
const Byte OurDeviceInfoLen = sizeof(OurDeviceInfo);

/* "Standard" class name for our demo is IrDemo with attribute of
   IrDA:IrLMP:LSAPSel
*/
static Byte irdemoQuery[] = {6,'I','r','D','E','M','O',18,'I','r','D','A',':','I','r','L','M','P',':',
                            'L','s','a','p','S','e','l'};
Int16 irdemoQuerySize = sizeof(irdemoQuery);

/* Result for IrDemo */
Byte irdemoResult[] = {
    0x01,                    /* Type for Integer is 1 */
    0x00,0x00,0x00,0x02      /* Assumed Lsap */
};
/* IrDEMO attribute */
IrIasAttribute irdemoAttribs = {
   (BytePtr)"IrDA:IrLMP:LsapSel",18,0,(BytePtr)irdemoResult, sizeof(irdemoResult),0
};
static IrIasObject irdemoObject =
{
  (BytePtr)"IrDEMO",6,1,
  (IrIasAttribute*)&irdemoAttribs
};


#define printf PalmOS_printf


/*
   MainFormEnableControl:
   Parms:   formP form pointer,
            object id of button to enable/disable, 
            true for enable | false to disable
   Return:  none
*/
static void
MainFormEnableControl(FormPtr formP, Word objectId, Boolean enabled)
{
  // Get object index from form
  Word objectIndex = FrmGetObjectIndex(formP,objectId);

  // get ptr to control
  ControlPtr controlP = (ControlPtr) FrmGetObjectPtr (formP, objectIndex);

  // set enabled
  CtlSetEnabled(controlP, enabled);

  if (enabled) 
  {
    CtlShowControl (controlP);
  }
  else
  {
    CtlHideControl (controlP);
  }
}



/*
   MainFormEventHandler:
   Parms:   pEvent   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormEventHandler (EventPtr eventP)
{
   Boolean  handled = false;
   
   switch (eventP->eType)
   {
      case menuEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormMenuHandler (formP, eventP);
         break;
      }
      
      case ctlSelectEvent:
      {
      // A control button was pressed.
         FormPtr formP = FrmGetActiveForm ();
         handled = MainFormButtonHandler (formP, eventP);
         break;
      }
      
      case frmOpenEvent:
      {
         FormPtr formP = FrmGetActiveForm ();
         MainFormInit (formP);
         FrmDrawForm (formP);
         handled = true;
         break;
      }
      
      default:
      {
         break;
      }
   }
   return handled;
}

/*
   MainFormInit:
   Initialize the main form.
   Parms:   formP - pointer to main form.
   Return:  none
*/
void
MainFormInit (FormPtr /*formP*/)
{
}

/*
   MainFormMenuHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            command  - command to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormMenuHandler (FormPtr /*formP*/, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.menu.itemID)
   {
      case AboutAbout:
         FrmCustomAlert(InfoAlert,"IrDemo 1999\nW.F. Ableson","","");
         handled = true;
         break;      
   }

   return handled;
}

/*
   MainFormButtonHandler:
   Handle a command sent to the main form.
   Parms:   formP    - form handling event.
            eventP   - event to be handled.
   Return:  true  - handled (successfully or not)
            false - not handled
*/
Boolean
MainFormButtonHandler (FormPtr formP, EventPtr eventP)
{
   Boolean handled = false;
   
   switch (eventP->data.ctlEnter.controlID)
   {
      case MainStartButton:   // User selected Start button.
      {
         MainFormStart (formP);
         handled = true;
         break;
      }

      case MainConnectButton:   // User selected Connect button.
      {
         MainFormConnect (formP);
         handled = true;
         break;
      }

      case MainSendButton:   // User selected Send button.
      {
         MainFormSend (formP);
         handled = true;
         break;
      }

      case MainFinishButton:   // User selected Finish button.
      {
         MainFormFinish();
         handled = true;
         break;
      }
  }
  return handled;
}

/*
   MainFormStart:
   Handle the main form's Connect command.
   Parms:   formP    - form handling event.
   Return:  none
*/
void
MainFormStart (FormPtr /*formP*/)
{
   Err e;
   long lCounter = 0;
   long lTimeout = 10;
  
   // load the Ir Library
   e = SysLibFind(irLibName,&refNum);
   if (e)
   {
      printf("Failed To Load Ir Library");
      return;
   }
   else
   {
      printf("IrLoad Success!");
      e = IrOpen ( refNum, irOpenOptSpeed9600 );
      if (e)
      {
         printf("Failed To Open");
         return;
      }
      else
      {
         printf("IrOpen Success!");
         e = IrBind(refNum,&IrCon,IrHandler);
         if (e)
         {
            printf("Failed To Bind");
            IrClose( refNum );
            return;
         }
         else
         {
            printf("IrBind Success!");
            if (IrSetDeviceInfo( refNum, OurDeviceInfo ,OurDeviceInfoLen ) !=
                         IR_STATUS_SUCCESS)
            {
               IrUnbind( refNum, &IrCon );
               IrClose( refNum );
               printf("IrSetDeviceInfo Failed!" );
               return;
            }
            else
            {
               printf("IrSetDeviceInfo Success!" );
               // set up IAS information about ourselves
               IrIAS_SetDeviceName( refNum, OurDeviceName,OurDeviceNameLen);
               IrIAS_Add( refNum, &irdemoObject);
               IrInitializedYet = 1;
               return;
            }  // set device info
         } // bind
      } // open
   } // libfind
}


/*
   MainFormConnect:
   Handle the main form's Connect command.
   Parms:   formP    - form handling event.
   Return:  none
*/
void
MainFormConnect (FormPtr /*formP*/)
{
   EventType evtExit;
   long lCounter = 0;
   const long lTimeout = 50;
   IrStatus irStat;

   if (IrInitializedYet == 0 ) return;
   // initiate a discovery and irlap connection
   while ( ++lCounter <= lTimeout)
   {
      irStat = IrDiscoverReq( refNum, &IrCon ); 
      switch (irStat)
      {
         case IR_STATUS_MEDIA_BUSY: 
            printf("Media Busy");
            continue;
         case IR_STATUS_FAILED:
            printf("Failed in Discovery");
            IrUnbind( refNum, &IrCon );
            IrClose( refNum );
            FrmCustomAlert(ErrorAlert,"Failed to Discover.  Ending Application","","");
            MemSet(&evtExit, sizeof(EventType), 0);
            evtExit.eType = appStopEvent;
            EvtAddEventToQueue(&evtExit);
            return;
         case IR_STATUS_PENDING:
            // this is the one we want!
            // at this point we need to wait for the discovery process to complete ...
            printf("Discovery Pending!!!");
            return;
      }
   } // while
}
   

/*
   MainFormSend:
   Handle the main form's WriteIr command.
   Parms:   formP    - form handling event.
   Return:  none
*/
void
MainFormSend (FormPtr /*formP*/)
{
   IrStatus irStat;
   CharPtr sendbuf = "Hello IrDemo!!";

   if (IrInitializedYet == 0) return;
   // let's send our message!
   irPack.len = StrLen(sendbuf);
   irPack.buff = (BytePtr) sendbuf;
   
	irStat = IrDataReq ( refNum, &IrCon, &irPack );

 	if ( irStat == IR_STATUS_PENDING ) 
   {
      printf("Packet on its way!");
   }
   else
   {
      printf("Error sending packet!");
   }
}


/*
   MainFormFinish:
   Handle the main form's Finish command.
   Parms:   none
   Return:  none
*/
void
MainFormFinish()
{
   EventType evtExit;
   if (IrInitializedYet == 1 ) 
   {
      // close Ir
      IrUnbind( refNum, &IrCon );
      IrClose( refNum );
   }
   MemSet(&evtExit, sizeof(EventType), 0);
   evtExit.eType = appStopEvent;
   EvtAddEventToQueue(&evtExit);
   
}



/*
   MainFormVersion:
   Handle the main form's version command.
   Parms:   formP    - form handling event (not used).
   Return:  true     - handled (successfully or not)
            false    - not handled
*/
void
MainFormVersion (FormPtr /*formP*/)
{
}
   
/*
   IrHandler:
   This function is installed by IrBind to handle the callbacks from
   the ir stack.  Examining pCBParms->event allows us to handle the call
   appropriately.  A good design practice is to move handling of each event
   to a helper function.

   Please note that with GCC, the CALLBACK_* macros
   are needed.

   Parms:   IrConnect *
            IrCallBackParms *
   Returns: void
*/
static void IrHandler(IrConnect * /*pCon*/,IrCallBackParms * pCBParms)
{
   long lCounter = 0;
   const long lTimeout = 5;
 
   CALLBACK_PROLOGUE 

   switch (pCBParms->event)
   {
      case LEVENT_DATA_IND:
         printf("Data Received!");
         DisplayRxBuff(pCBParms);
         break;
      case LEVENT_DISCOVERY_CNF:
         printf("Discovery Complete");
         OnDiscovery(pCBParms);
         break;
      case LEVENT_LAP_CON_CNF:
         printf("LAP Conn Confirmed!");
         OnLapConfirmed();
         break;
      case LEVENT_LAP_CON_IND:
         printf("LAP Connection Established!");
         break;
      case LEVENT_LAP_DISCON_IND:
         printf("LAP Connection Lost!!!");
         break;
      case LEVENT_LM_CON_CNF:
         printf("LMP Session Established!");
         break;     
      case LEVENT_LM_CON_IND:
         printf("LMP Session Requested");
         OnLMPRequest();
         break;     
      case LEVENT_LM_DISCON_IND:
         printf("LMP Connection Dropped");
         break;     
      case LEVENT_PACKET_HANDLED:
         printf("Packet Handled");
         break;
      case LEVENT_STATUS_IND:
         switch (pCBParms->status)
         {
            case IR_STATUS_SUCCESS:              /* Successful and complete */
               printf( "IR_STATUS_SUCCESS" );
               break;
            case IR_STATUS_FAILED:                      /* Operation failed */
               printf( "IR_STATUS_FAILED" );
               break;
            case IR_STATUS_PENDING:     /* Successfully started but pending */
               printf( "IR_STATUS_PENDING" );
               break;
            case IR_STATUS_DISCONNECT:                 /* Link disconnected */
               printf( "IR_STATUS_DISCONNECT" );
               break;
            case IR_STATUS_NO_IRLAP:          /* No IrLAP Connection exists */
               printf( "IR_STATUS_NO_IRLAP" );
               break;
            case IR_STATUS_MEDIA_BUSY:                  /* IR Media is busy */
               iMediaBusy = 1;
               break;
            case IR_STATUS_MEDIA_NOT_BUSY:          /* IR Media is not busy */
               iMediaBusy = 0;
               break;
            case IR_STATUS_NO_PROGRESS:
               printf("IR_STATUS_NO_PROGRESS" );
               break;
            case IR_STATUS_LINK_OK:
               printf( "IR_STATUS_LINK_OK" );
               break;
            default:
               printf("other status %d",pCBParms->status);
               break;
         }
         break;
      case LEVENT_TEST_CNF:
         printf("callback - LEVENT_TEST_CNF");
         break;
      case LEVENT_TEST_IND:
         printf("callback - LEVENT_TEST_IND");
         break;
   }

   CALLBACK_EPILOGUE

}


/*
   IASHandler:
   Callback function for IAS query results.  This function is called when an
   IAS query has completed.  The parameter, irStat can be either 
   IR_STATUS_SUCCESS or IR_STATUS_DISCONNECTED.  If the query has completed,
   the status is IR_STATUS_SUCCESS.  IR_STATUS_DISCONNECTED indicates that the
   query could not finish due to a connection problem.  

   Assuming IR_STATUS_SUCCESS, we then retrieve the results.  First we
   determine the datatype of a query result and then retrieve it.... At the 
   moment, this implementation is rather blind to the "state" of things,
   rather, it determines, based on datatype, what to do with the results.  For
   example, if the result is of type IAS_ATTRIB_INTEGER, we assume that it is 
   the address of the service we were looking for (ie IrLPT or IrCOMM, etc.)
   If the data type is IAS_USER_STRING, we assume that it is the response to
   a request for the device name.

   Please note that with GCC, the CALLBACK_* macros are needed.  
   
   Parms:   IrStatus
   Returns void
*/
void IASHandler(IrStatus irStat)
{
   char szBuf[50];
   IrStatus irStat2;
   long lCounter = 0;
   const long lTimeout = 10;

   CALLBACK_PROLOGUE

   StrPrintF(szBuf,"irStat %d retcode %d type %d",
             irStat,clientQuery.retCode,IrIAS_GetType(&clientQuery));
   printf(szBuf);
   switch (irStat)
   {
      case IR_STATUS_SUCCESS:
         switch ( IrIAS_GetType(&clientQuery) )
         {
            case IAS_ATTRIB_MISSING:
               printf("Attribute is Missing?!");
               break;
            case IAS_ATTRIB_INTEGER:
               printf("Get Integer Value");
               IrCon.rLsap = IrIAS_GetIntLsap(&clientQuery);
               irPack.len = 0;
               // we have the address for the service we want to connect to
               // let's establish the LMP session
               printf("Go Ahead and Make Connection!");

               // we want an LMP session (as opposed to TinyTP)
               IrSetConTypeLMP(&IrCon);

               irPack.len = 0;   // we are using an empty packet (but a valid one!)

               // because we could wind up with a media busy, we wrap
               // our request in a "retry" loop
               lCounter = 0;
               while ( ++lCounter <= lTimeout)
               {
                  irStat2 = IrConnectReq( refNum, &IrCon, &irPack, 2 ); 
                  switch (irStat2)
                  {
                     case IR_STATUS_MEDIA_BUSY: 
                        printf("Media Busy");
                        continue;
                     case IR_STATUS_FAILED:
                        FrmCustomAlert(ErrorAlert,"Failed in LMP Connect Attempt","",""); 
                        // todo error handling here
                        return;
                     case IR_STATUS_NO_IRLAP:
                        FrmCustomAlert(ErrorAlert,"IrLAP is missing!","","");            
                        // todo error handling here
                        return;
                     case IR_STATUS_PENDING:
                        // this is the one we want!
                        // at this point we need to wait for the connect process to complete ...
                        printf("Connect Lmp Pending!!!");
                        return;
                  }
               } // while

               break;
            case IAS_ATTRIB_OCTET_STRING:
               // handle any octet string we may receive from IAS
               break;
            case IAS_ATTRIB_USER_STRING:
               // this user string is from our DeviceName request
               // pull it in and display
               StrCopy(szDeviceName,(char *)IrIAS_GetUserString(&clientQuery));
               printf(szDeviceName);

               // todo: verify that this is the device we are looking for ...

               // now that we have the device we want
               // we need to determine how to connect to it, ie. what lsap?
               IrIAS_StartResult(&clientQuery);
               clientQuery.result = queryResult;
               clientQuery.resultBufSize = sizeof(queryResult);
               clientQuery.callBack = IASHandler;
               clientQuery.queryBuf = irdemoQuery;   
               clientQuery.queryLen = irdemoQuerySize;
               IrIAS_Query(refNum, &clientQuery);
               break;
            case IAS_ATTRIB_UNDEFINED:
            default:
               printf("Undefined Attribute??");
               break;
         }
         break;
      default:
         printf("IASHandler Service Not Found");
         break;
   }


   CALLBACK_EPILOGUE
}


/*
   DisplayRxBuff:
   Display the data received.  This is just a helper function.
   Parms:   pIrCallback Parms
   Retrns:  none
*/
static void DisplayRxBuff(IrCallBackParms * pCBParms )
{
   char szBuf[25];
   Word i=0;
   while (i < pCBParms->rxLen)
   {
      MemSet(szBuf,sizeof(szBuf),0x00);
      if (pCBParms->rxLen - i > (Word) sizeof(szBuf))
      {
         // copy full buffer at a time
         MemMove(szBuf,pCBParms->rxBuff,sizeof(szBuf));
         i+= sizeof(szBuf);
      }
      else
      {
         // copy balance of buffer
         MemMove(szBuf,pCBParms->rxBuff,pCBParms->rxLen - i);
         i = pCBParms->rxLen;
      }
      printf(szBuf);
   }
}


/*
   OnDiscovery:
   Get address of discovered devices
   Parms:   pIrCallback Parms
   Retrns:  none
*/
static void OnDiscovery(IrCallBackParms * pCBParms)
{
   long lCounter = 0;
   const long lTimeout = 10;
   IrStatus irStat;
   
   // check for a valid device
   if (pCBParms->deviceList->nItems == 0)
   {
      printf("No Devices Found!");
      return;
   }

   // at least one device has been found, we will
   // assume that the first (and probably only!) device
   // found is the one we want.
   g_irDevice = pCBParms->deviceList->dev[0].hDevice;
   printf("Found %d.%d.%d.%d",g_irDevice.u8[0],g_irDevice.u8[1],
            g_irDevice.u8[2],g_irDevice.u8[3]);

   // let's make an IrLap connection to this address
   while ( ++lCounter <= lTimeout)
   {
      irStat = IrConnectIrLap( refNum, g_irDevice ); 
      switch (irStat)
      {
         case IR_STATUS_MEDIA_BUSY: 
            printf("IrLap Media Busy");
            continue;
         case IR_STATUS_FAILED:
            printf("Failed in IrConnectIrLap" );
            return; 
         case IR_STATUS_PENDING:
            // this is the one we want!
            // at this point we need to wait for the connect process to complete ...
            printf("Connect Lap Pending!!!");
            return;
      }
   } // while
}

/*
   OnLapConfirmed:
   Continue with connection Process by getting device name
   Parms:   pIrCallback Parms
   Retrns:  none
*/
static void OnLapConfirmed()
{
   // initiate query for remote service we are interested in
   // this first query will provide the remote device name
   // as defined by the device's IAS
   IrIAS_StartResult(&clientQuery);
   clientQuery.result = queryResult;
   clientQuery.resultBufSize = sizeof(queryResult);
   clientQuery.callBack = IASHandler;
   clientQuery.queryBuf = irGetQuery;
   clientQuery.queryLen = irGetQuerySize;
   IrIAS_Query(refNum, &clientQuery);
}

/*
   OnLMPRequest:
   Respond to request for LMP Session
   Parms:   pIrCallback Parms
   Retrns:  none
*/
static void OnLMPRequest()
{
   IrStatus irStat;	
   // another device wants connection,
   // so let's accept the invitation!
   irStat = IrConnectRsp(refNum, &IrCon, &irPack, 2);
   if (irStat != IR_STATUS_PENDING)
   {
      printf("Trouble Accepting LMP");
   }

}



/*
   PalmOS_printf:
   Implementation of c run time function printf.  This is meant for debug
   purposes ONLY.  This function keeps a static vertical offset and displays
   the string on the next "line".  When the bottom of the screen is reached,
   we go back to the top of the screen.  This function relies on the Palm function
   StrVPrintf and is subject to the limitations of that function.
   This is a poor man's debugging scheme, feel free to roll your own...
*/   
void PalmOS_printf(char * pszFormat, ...)
{
   static int iOffset = 105;
   const int iDelta = 10;
   static long lCounter=1;
   char szBuf[200]; // maximum size ... should not be an issue
   char *szBlanks = "                                                                             ";
   va_list args;
  
   // process variable arguments through existing function StrVPrintf()
   va_start(args,pszFormat);
   StrVPrintF(szBuf,pszFormat,args);
   va_end(args);
   
   // clear the line, then "draw" our text
   WinDrawChars(szBlanks,StrLen(szBlanks),0,iOffset);
   WinDrawChars(szBuf,StrLen(szBuf),0,iOffset);

   // move to next line.
   iOffset += iDelta;

   // check for "wrap"
   if (iOffset > 145) {
      iOffset = 105;
   }

}
