/******************************************************************************
 * File: daytime_berkeley.c
 * Author: W.F. Ableson, frank@cfgsolutions.com
 * Description: Example application for UDP Programming, ported to Berkeley stye
 *****************************************************************************/

#include <PalmOS.h>
#include "daytime_berkeleyRsc.h"

// needed for Berkeley macros
#include <sys_socket.h>

/***********************************************************************
 *
 *	Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *	Internal Structures
 *
 ***********************************************************************/
typedef struct 
	{
	UInt8 replaceme;
	} daytime_palmPreferenceType;

typedef struct 
	{
	UInt8 replaceme;
	} daytime_palmAppInfoType;

typedef daytime_palmAppInfoType* daytime_palmAppInfoPtr;


/***********************************************************************
 *
 *	Global variables
 *
 ***********************************************************************/
//static Boolean HideSecretRecords;
UInt16 errno;        // needed by NetSocket.c

/***********************************************************************
 *
 *	Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			'DYPM'	// register your own at http://www.palmos.com/dev/creatorid/
#define appVersionNum			0x01
#define appPrefID				   0x00
#define appPrefVersionNum		0x01

// Define the minimum OS version we support (3.0 for now).
#define ourMinVersion	sysMakeROMVersion(3,0,0,sysROMStageRelease,0)
#define kPalmOS10Version	sysMakeROMVersion(1,0,0,sysROMStageRelease,0)

/***********************************************************************
 *
 *	Prototypes
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID);
void GetDayTime(Char * host);
Char * GetTargetServer(Char * title,Char * prompt,Char * defaultvalue);
void MsgBox(Char * s);
void ErrBox(Char * s);



/***********************************************************************
 *
 *	Internal Functions
 *
 ***********************************************************************/

void MsgBox(Char * s)
{
   FrmCustomAlert(InfoAlert,s,NULL,NULL);
}


void ErrBox(Char * s)
{
   FrmCustomAlert(ErrorAlert,s,NULL,NULL);
}


/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion <= kPalmOS10Version)
				{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return sysErrRomIncompatible;
		}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}


/*
utilities for handling user input
*/
Char inputboxvalue[100];
Char * GetTargetServer(Char * title,Char * prompt,Char * defaultvalue)
{
FormPtr pfrm = 0;
Char * ret = 0;
ControlPtr pctl = 0;
FieldPtr pfield = 0;
MemHandle memH = 0;
Char * ptr = 0;

	StrCopy(inputboxvalue,defaultvalue);
	pfrm = FrmInitForm(TargetServerForm);
	if (!pfrm) return NULL;
   pctl = FrmGetObjectPtr(pfrm,FrmGetObjectIndex(pfrm,TargetServerPromptLabel));
	if (pctl == NULL) goto leave;
	CtlSetLabel(pctl,prompt);
	FrmSetTitle(pfrm,title);
   pfield = FrmGetObjectPtr(pfrm,FrmGetObjectIndex(pfrm,TargetServerValueField));
   if (!pfield) goto leave;

	// need to allocate a buffer here for text field
	memH = MemHandleNew(100);
	if (memH == 0) goto leave;
   ptr = (Char *) MemHandleLock(memH);
   StrCopy(ptr,"");
   MemHandleUnlock(memH);
  	FldSetTextHandle(pfield,memH);	
	if (defaultvalue)
	{
		// set the field to this value
		memH = FldGetTextHandle(pfield);
		ptr = (Char *) MemHandleLock(memH);
	   if (ptr == 0) goto leave;	
      StrCopy(ptr,defaultvalue);
		MemHandleUnlock(memH);
    	FldSetTextHandle(pfield,memH);
	}
	if (TargetServerOKButton == FrmDoDialog(pfrm))
	{
      if (FldGetTextLength(pfield))
      {
         memH = FldGetTextHandle(pfield);
         ptr = (Char *) MemHandleLock(memH);
         if (ptr)
         {
      	   if (StrLen(ptr))
            {
      			StrCopy(inputboxvalue,ptr);
      			ret = inputboxvalue;
            }
         }
         MemHandleUnlock(memH);
      }
      else
      {
         StrCopy(inputboxvalue,"");
         ret = inputboxvalue;
      }
	}
leave:
	if (memH)
	{
	   FldSetTextHandle(pfield,0);
		MemHandleFree(memH);
	}
	FrmEraseForm(pfrm);
	FrmDeleteForm(pfrm);
	return ret;
}








/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr frmP)
{
   // initialize the daytime server to be the localhost
   StrCopy(inputboxvalue,"localhost");
}


/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
	FormPtr frmP;

	switch (command)
		{
		case MainOptionsAboutdaytime_berkeley:
			MenuEraseStatus(0);					// Clear the menu status from the display.
			frmP = FrmInitForm (AboutForm);
			FrmDoDialog (frmP);					// Display the About Box.
			FrmDeleteForm (frmP);
			handled = true;
			break;

		}
	
	return handled;
}

/*
handle the button select event
*/
static Boolean MainFormHandleButton(FormPtr formP,EventPtr eventP)
{
   Boolean handled = false;
   UInt16 index = FrmGetObjectIndex (formP, MainGoButton);
   FieldPtr fieldP = (FieldPtr) FrmGetObjectPtr (formP, index);
   Char serveraddress[100];
         
   switch (eventP->data.ctlEnter.controlID)
   {
      
      case MainGoButton:
      {
         StrCopy(serveraddress,GetTargetServer("Daytime Server","Name of Daytime Server",inputboxvalue));
         if (StrLen(serveraddress))
         {
            GetDayTime(serveraddress);
         }
         handled = true;
         break;
      }
   }
   return handled;
}




/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
	Boolean handled = false;
	FormPtr frmP;

	switch (eventP->eType) 
		{
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			handled = true;
			break;
			
      case ctlSelectEvent:
         frmP = FrmGetActiveForm ();
         return MainFormHandleButton(frmP,eventP);

		case frmUpdateEvent:
			// To do any custom drawing here, first call FrmDrawForm(), then do your
			// drawing, and then set handled to true.
			break;

		default:
			break;
		
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
	daytime_palmPreferenceType prefs;
	UInt16 prefsSize;

	// Read the saved preferences / saved-state information.
	prefsSize = sizeof(daytime_palmPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
		noPreferenceFound)
		{
		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
	daytime_palmPreferenceType prefs;

	// Write the saved preferences / saved-state information.  This data 
	// will saved during a HotSync backup.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
		
	// Close all the open forms.
	FrmCloseAllForms ();
}


/***********************************************************************
 *
 * FUNCTION:    daytime_palmPalmMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static UInt32 daytime_palmPalmMain(UInt16 cmd, MemPtr /*cmdPBP*/, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;

		default:
			break;

		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	return daytime_palmPalmMain(cmd, cmdPBP, launchFlags);
}


/*
Handle the Daytime Query
*/
void GetDayTime(Char * host)
{
NetSocketRef sock = 0;
Err err = 0;
struct sockaddr_in	daytime_srv_addr;	/* server's Internet socket addr */
Int16 retval = 0;
UInt16 neterr;
struct hostent			*hp;
//struct hostent			tcp_host_info;	/* from gethostbyname() */
Char buf[200];
Char lbuf[200];
UInt16 addrlen;

   // attempt to find the Network Library
   err = SysLibFind ("Net.lib", &AppNetRefnum);

   // was the library found?
   if (!err)
   {
      // attempt to open the library
      err = NetLibOpen(AppNetRefnum,&neterr);
      if (err)
      {
         StrPrintF(buf,"Failed to open Network library.  Error Code is [%d]",neterr);
         ErrBox(buf);
         return;
      }
   }
   else
   {
      ErrBox("Failed to find Net.lib, cannot continue");
      return;
   }
   // create the socket, Berkeley style
   sock = socket(AF_INET,SOCK_DGRAM,0);
   if (sock != -1)
   {
      // prepare server address
      bzero((char *) &daytime_srv_addr, sizeof(daytime_srv_addr));
      // set to inet address family
      daytime_srv_addr.sin_family = AF_INET;
      // set the port to the daytime port
      daytime_srv_addr.sin_port = htons(13);      
      // find target host       
		hp = gethostbyname(host);
		if ( hp == NULL)
		{
		   StrPrintF(buf,"Failed to locate host [%s]",host);
		   ErrBox(buf);
		   close(sock);
		   NetLibClose(AppNetRefnum,true);
			return;
		}
		//tcp_host_info = *hp;
		bcopy(hp->h_addr, (char *) &daytime_srv_addr.sin_addr,hp->h_length);
      if (sendto(sock,lbuf,0,0,&daytime_srv_addr,sizeof(daytime_srv_addr)) == -1)
      {
         StrPrintF(buf,"Failed to send to daytime server, errno is [%x]",errno);
         ErrBox(buf);
         close(sock);
         NetLibClose(AppNetRefnum,true);
         return;
      }
      // let's receive the response from the daytime server
      addrlen = sizeof(daytime_srv_addr);
      bzero(lbuf,sizeof(lbuf));
      if (recvfrom(sock,lbuf,sizeof(lbuf),0,&daytime_srv_addr,&addrlen) == -1)
      {
         StrPrintF(buf,"Failed to receive from daytime server, errno is [%d]",errno);
         ErrBox(buf);
         close(sock);
         NetLibClose(AppNetRefnum,true);
         return;
      }
      // display response
      MsgBox(lbuf);
      close(sock);
      NetLibClose(AppNetRefnum,true);
      return;
   }
   else
   {
      StrPrintF(buf,"Unabled to Create Socket, ErrorCode [%x]",errno);
      ErrBox(buf);
   }
   NetLibClose(AppNetRefnum,true);
   return;
   
}