/******************************************************************************
 * File: http_palm.c
 * Author: W.F. Ableson, frank@cfgsolutions.com
 * Description: Example application for TCP Programming
 *****************************************************************************/

#include <PalmOS.h>
#include "http_palmRsc.h"


/***********************************************************************
 *
 *   Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *   Prototypes
 *
 ***********************************************************************/

Boolean TargetURLForm_HandleEvent(EventPtr event);
Char * GetTargetURL(Char * title,Char * prompt,Char * defaultvalue);

Int16 SendHTTPRequest(Char * host, Char * uri,Int16 port);
Int16 GetHTTPHeaderLine(NetSocketRef s, Char * buf);
Int16 GetHTTPHeader(NetSocketRef s, Int16 * len);
Int16 GetHTTPBody(NetSocketRef s, Int16 len,Char * data);

/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/
typedef struct 
	{
	UInt8 replaceme;
	} http_palmPreferenceType;

typedef struct 
	{
	UInt8 replaceme;
	} http_palmAppInfoType;

typedef http_palmAppInfoType* http_palmAppInfoPtr;


/***********************************************************************
 *
 *   Global variables
 *
 ***********************************************************************/
//static Boolean HideSecretRecords;
static UInt16 refNum = 0;
static NetSocketRef sock = 0;
static Err err = 0;
static UInt16 neterr = 0;
static NetSocketAddrINType sa;
static Int16 retval = 0;
static NetHostInfoBufType   hostbuf;
static NetHostInfoType *   pHost = NULL;
static Char buf[200];
static Char address[200];
static Char hostportion[200];
static Char uriportion[200]; 
static Int16 portnumber;
static UInt8 urlisvalid = 0;
/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
#define appFileCreator				 'htpm'
#define appVersionNum              0x01
#define appPrefID                  0x00
#define appPrefVersionNum          0x01

// Define the minimum OS version we support (2.0 for now).
#define ourMinVersion	sysMakeROMVersion(2,0,0,sysROMStageRelease,0)


/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/
static void MsgBox(Char * message)
{
   FrmCustomAlert(InfoAlert,message,NULL,NULL);
}   

static void ErrBox(Char * message)
{
   FrmCustomAlert(ErrorAlert,message,NULL,NULL);
}   

/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < ourMinVersion)
				{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return sysErrRomIncompatible;
		}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}


/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr /*frmP*/)
{
}


/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
   FormPtr frmP;
   Char *p;
   Char *p2;
   Char lbuf[100];
   
	switch (command)
		{
		case OptionsAbouthttp_palm:
			MenuEraseStatus(0);					// Clear the menu status from the display.
			frmP = FrmInitForm (AboutForm);
			FrmDoDialog (frmP);					// Display the About Box.
			FrmDeleteForm (frmP);
			handled = true;
			break;

      case OptionsLoadLibrary:
         if (refNum != 0)
         {
            MsgBox("Library Already Loaded!");
         }
         else
         {
            // attempt to find the Network Library
            err = SysLibFind ("Net.lib", &refNum);

            // was the library found?
            if (!err)
            {
               // attempt to open the library
               err = NetLibOpen(refNum,&neterr);
               if (!err)
               {
                  MsgBox("Network library has been loaded & opened successfully");
               }
               else
               {
                  StrPrintF(buf,"Failed to open Network library.  Error Code is [%d]",neterr);
                  ErrBox(buf);
               }
            }
            else
            {
               ErrBox("Failed to find Net.lib, cannot continue");
            }
         }
         break;
      case OptionsSetDestination:
tryagain:      
         StrCopy(address, GetTargetURL("http_palm","Enter a url using a domain name.\rIP Address format is not supported.\rA port number is permitted.",address));
         // crack the address into host and "page"
         MemSet(hostportion,sizeof(hostportion),0x00);
         MemSet(uriportion,sizeof(uriportion),0x00);   
         //default the port to 80
         portnumber = 80;
         p = (Char *) StrStr(address,"http://");
         if (p)
         {
            // jump past protocol specifier
            p+= StrLen("http://");
            // look for optional port number
            p2 = (Char*) StrStr(p,":");
            if (p2)
            {
               // pull out the host and then the port
               StrNCopy(hostportion,p,p2-p);
               p = (Char*) StrStr(p2,"/");
               if (p)
               {
                  // "/" found, pull out port
                  StrNCopy(lbuf,p2,p-p2);
                  p2++;
                  portnumber = StrAToI(p2);
                  // uri portion is the balance of the string
                  StrCopy(uriportion,p);
               }
               else
               {
                  // balance of the address is the port and the uri is "/"
                  p2++;
                  portnumber = StrAToI(p2);
                  StrCopy(uriportion,"/");
               }                  
            }   
            else
            {
               // look for "/"
               p2 = (Char*) StrStr(p,"/");
               if (p2)
               {
                  // "/" found, pull out host name
                  StrNCopy(hostportion,p,p2-p);
                  StrCopy(uriportion,p2);
               }
               else
               {
                  StrCopy(hostportion,p);
                  StrCopy(uriportion,"/");
               }                  
            }
            urlisvalid = 1;
            MsgBox(hostportion);
            MsgBox(uriportion);
            StrPrintF(lbuf,"%d",portnumber);
            MsgBox(lbuf); 
         }
         else
         {
            ErrBox("Please enter a url of the form:\nhttp://hostname/page");
         }
         break;
      case OptionsPerformHttpOperation:
         if (urlisvalid == 1)
         {
            SendHTTPRequest(hostportion, uriportion,portnumber);
         }
         else
         {
            MsgBox("Please provide a target URL");
            goto tryagain;
         }
         break;
      case OptionsCloseLibrary:
         if (refNum == 0)
         {
            MsgBox("Library not loaded!");
         }
         else
         {
            err = NetLibClose(refNum,true);
            if (err)
            {
               StrPrintF(buf,"Errors during NetLibClose.  Error Code is [%d]",err);
               ErrBox(buf);
            }
            else
            {
               MsgBox("Network Library has been closed");
            }
            refNum = 0;
         }
         break;
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
   Boolean handled = false;
   FormPtr frmP;

	switch (eventP->eType) 
		{
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			handled = true;
			break;

		default:
			break;
		
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;
         case TargetURLForm:
            FrmSetEventHandler(frmP, TargetURLForm_HandleEvent);
            break;
			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

   } while (event.eType != appStopEvent);
   if (refNum) NetLibClose(refNum,true);
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
    http_palmPreferenceType prefs;
    UInt16 prefsSize;

	// Read the saved preferences / saved-state information.
	prefsSize = sizeof(http_palmPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
		noPreferenceFound)
		{
		}
	
   return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
   http_palmPreferenceType prefs;

	// Write the saved preferences / saved-state information.  This data 
	// will be backed up during a HotSync.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
		
	// Close all the open forms.
	FrmCloseAllForms ();
}


/***********************************************************************
 *
 * FUNCTION:    http_palmPalmMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static UInt32 http_palmPalmMain(UInt16 cmd, MemPtr /*cmdPBP*/, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;

		default:
			break;

		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return http_palmPalmMain(cmd, cmdPBP, launchFlags);
}




/*
utility functions for obtaining data from the user
*/
Char inputboxvalue[100];
Char * GetTargetURL(Char * title,Char * prompt,Char * defaultvalue)
{
FormPtr pfrm = 0;
Char * ret = 0;
ControlPtr pctl = 0;
FieldPtr pfield = 0;
MemHandle memH = 0;
Char * ptr = 0;


	StrCopy(inputboxvalue,defaultvalue);
	pfrm = FrmInitForm(TargetURLForm);
	if (!pfrm) return NULL;
   pctl = FrmGetObjectPtr(pfrm,FrmGetObjectIndex(pfrm,TargetURLPromptLabel));
	if (pctl == NULL) goto leave;
	CtlSetLabel(pctl,prompt);
	FrmSetTitle(pfrm,title);
   pfield = FrmGetObjectPtr(pfrm,FrmGetObjectIndex(pfrm,TargetURLValueField));
   if (!pfield) goto leave;

	// need to allocate a buffer here for text field
	memH = MemHandleNew(100);
	if (memH == 0) goto leave;
   ptr = (Char *) MemHandleLock(memH);
   StrCopy(ptr,"");
   MemHandleUnlock(memH);
  	FldSetTextHandle(pfield,memH);	
	if (defaultvalue)
	{
		// set the field to this value
		memH = FldGetTextHandle(pfield);
		ptr = (Char *) MemHandleLock(memH);
	   if (ptr == 0) goto leave;	
      StrCopy(ptr,defaultvalue);
		MemHandleUnlock(memH);
    	FldSetTextHandle(pfield,memH);
	}
	if (TargetURLOKButton == FrmDoDialog(pfrm))
	{
      if (FldGetTextLength(pfield))
      {
         memH = FldGetTextHandle(pfield);
         ptr = (Char *) MemHandleLock(memH);
         if (ptr)
         {
      	   if (StrLen(ptr))
            {
      			StrCopy(inputboxvalue,ptr);
      			ret = inputboxvalue;
            }
         }
         MemHandleUnlock(memH);
      }
      else
      {
         StrCopy(inputboxvalue,"");
         ret = inputboxvalue;
      }
	}
leave:
	if (memH)
	{
	   FldSetTextHandle(pfield,0);
		MemHandleFree(memH);
	}
	FrmEraseForm(pfrm);
	FrmDeleteForm(pfrm);
	return ret;
}


/*
button handler
*/
static Boolean TargetURLForm_cmdOK_OnSelect(EventPtr /* event */)
{
	// Insert code for cmdOK
	FrmReturnToForm(0);
	return true;
}

/*
event handler for user input form
*/
Boolean TargetURLForm_HandleEvent(EventPtr event)
{
	FormPtr form;
	Boolean handled = false;

	switch (event->eType)
	{
		case ctlSelectEvent:
			switch (event->data.ctlSelect.controlID)
			{
				// cmdOK receives an event
				case TargetURLOKButton:
					handled = TargetURLForm_cmdOK_OnSelect(event);
					break;
			}
			break;
		case frmOpenEvent:
			// Repaint form on open
			form = FrmGetActiveForm();
			FrmDrawForm(form);
			handled = true;
			break;
		default:
			break;
	}
	return handled;
}

/*
read a line from the web server
*/
Int16 GetHTTPHeaderLine(NetSocketRef s, Char * buf)
{
static NetFDSetType readFDs;
Int16 n;
UInt16 timeoutcount=0;
UInt32 rc=0;
Char c;
Int16 index=-1;


   while (1)
   {
      netFDZero(&readFDs);
      netFDSet(s,&readFDs);
      switch (n = NetLibSelect(refNum,netFDSetSize,&readFDs,NULL,NULL,SysTicksPerSecond() * 1,&neterr))
      {
         case -1:
            // handle error!
            return -1;
         case 0:
            // timeout
            MsgBox("timeout");
            return -1;
         default: 
            if (netFDIsSet(s,&readFDs))
            {
               switch  (NetLibReceive(refNum,s,&c,1,0,NULL,NULL,SysTicksPerSecond() * 1,&neterr))
               {
                  case 0:
                  case -1:
                     // done....
                     return 999;
                  case 1:
                     // check for newline
                     switch (c)
                     {
                        case 0x0d:  // eat this
                           break;   
                        case 0x0a:
                           // term this line
                           buf[++index] = 0x00;
                           MsgBox(buf);
                           return 0;
                        default:
                           buf[++index] = c;
                           break;
                     }
                     break;
               } //switch
            }
            else
            {
               // what happened... we were told that a socket was ready for reading, but not our socket??
               // this should NEVER happen!
               MsgBox("????");
            }
            break;
      } // switch
   } // while (forever)
   return 999;
}   
   



// we want to read through the response until we see the http response, if that is ok
// we need to get the content-length field.. then eat the header until we get the \n\n

Int16 GetHTTPHeader(NetSocketRef s, Int16 * len)
{
Int16 rc;
Int16 ret;
Char buf[500];
Char *p;
Int16 blankcount = 0;
Boolean bFoundLen = false;

   do
   {
      MemSet(buf,sizeof(buf),0x00);
      rc = GetHTTPHeaderLine(s,buf);
      if (rc == 0)
      {
         if (!StrNCompare(buf,"HTTP/1.1",8))
         {
            // get the return code
            ret = StrAToI(&buf[9]);
            if (ret == 200)
               ret = 0;
         }
         // look for a header line containing the length of the returned results
         p = (Char *) StrStr(buf,"Content-Length:");
         if (p)
         {
            p += 16;
            *len = (Int16) StrAToI(p);
            bFoundLen = true;
         }
         if (!p)
         {  
            // look to eat the pair of blank lines
            if (buf[0] == 0x00)
            {
               if (!bFoundLen)
               {
                  *len = -1;
               }
               return ret;
            }
         }
      }
   } while (rc == 0);   
   
   return ret;
}

/*
receive the http response body
*/
Int16 GetHTTPBody(NetSocketRef s, Int16 len,Char * data)
{
Int16 l;
Char c;
Int16 n;
static NetFDSetType readFDs;


   l = 0;

   while (1)
   {
      netFDZero(&readFDs);
      netFDSet(s,&readFDs);
      switch (n = NetLibSelect(refNum,netFDSetSize,&readFDs,NULL,NULL,SysTicksPerSecond() * .5,&neterr))
      {
         case -1:
            // handle error!
            ErrBox("error reading in GetHTTPBody?");
            return -1;
         case 0:
            // timeout
            return 0;
         case 1: 
            switch  (NetLibReceive(refNum,s,&c,1,0,NULL,NULL,SysTicksPerSecond() * .5,&neterr))
            {
               case 0:
               case -1:
                  // done....
                  return l;
               case 1:
                  if (l < len)
                  {
                     data[l++] = c;
                  }
                  else
                  {
                     ErrBox("Buffer size insufficient!");
                     return l;
                  }
                  break;
            }
            break;
      }
   }
   
   return -1;
}

/*
send a request to a web server
paramaters: hostname, page, port number
*/
Int16 SendHTTPRequest(Char * host, Char * uri,Int16 port)
{
static NetFDSetType writeFDs;   
Int16 n,rc,contentlength;
Char lbuf[200];
static MemHandle memH;
static Char * memP = NULL;

   sock = NetLibSocketOpen(refNum,netSocketAddrINET, netSocketTypeStream,netSocketProtoIPTCP, 1000,&neterr); 
   if (sock != -1)
   {
		MemSet(&sa,sizeof(sa),0x00);
		sa.family = netSocketAddrINET;
      pHost = NetLibGetHostByName(refNum,host,&hostbuf,SysTicksPerSecond() * 3,&neterr);
      if (pHost == NULL)
      {	
         StrPrintF(buf,"Failed to find host [%s]",host); 
         ErrBox(buf);
      }
      else
      {
         sa.addr = hostbuf.address[0];
			sa.port = NetHToNS(port); 
			retval = NetLibSocketConnect(refNum, sock, (NetSocketAddrType *) &sa,sizeof(sa),3 * SysTicksPerSecond(),&neterr);
			if (retval == -1)
			{
            StrPrintF(buf,"Unabled to Connect, ErrorCode [%d]",neterr);
            ErrBox(buf);
			}
			else
			{
            netFDZero(&writeFDs);
            netFDSet(sock,&writeFDs);
            switch (n = NetLibSelect(refNum,netFDSetSize,NULL,&writeFDs,NULL,SysTicksPerSecond() * 2,&neterr))
            {
               case -1:
                  // handle error!
                  ErrBox("error reading in SendHTTPRequest?");
                  return -1;
               case 0:
                  // timeout
                  ErrBox("timeout waiting to send in SendHTTPRequest?");
                  return -1;
               default: 
                  if (netFDIsSet(sock,&writeFDs))
                  {
                     // send request
                     StrPrintF(lbuf,"GET %s HTTP/1.1\r\n",uri);
                     rc = NetLibSend(refNum,sock,lbuf,StrLen(lbuf),0,NULL,0,SysTicksPerSecond() * 1,&neterr);
                     if (rc == -1)
                     {
                        StrPrintF(buf,"Error Sending [%d]",neterr);
                        ErrBox(buf);
                        return -1;
                     }
                     StrPrintF(lbuf,"Host: %s\r\n\r\n",host);
                     rc = NetLibSend(refNum,sock,lbuf,StrLen(lbuf),0,NULL,0,SysTicksPerSecond() * 1,&neterr);
                     if (rc == -1)
                     {
                        StrPrintF(buf,"Error Sending [%d]",neterr);
                        ErrBox(buf);
                        return -1;
                     }
                     // get the results!
                     MsgBox("Get results of request");
                     if (GetHTTPHeader(sock, &contentlength) == 0)
                     {
                        if (contentlength == 0) MsgBox("ugh");
                        StrPrintF(lbuf,"size is %d",contentlength);MsgBox(lbuf);
                        memH = MemHandleNew(contentlength+10);
                        if (memH)
                        {
                           memP = (Char *) MemHandleLock(memH);
                           if (memP)
                           {
                              MsgBox("Getting http body");
                              if (GetHTTPBody(sock,contentlength,memP) == 0)
                              {
                                 MsgBox(memP);
                              }
                              MemHandleUnlock(memH);
                              MemHandleFree(memH);
                           }
                           else
                           {
                              ErrBox("Unable To Lock Memory");
                           }
                        }
                        else
                        {
                           ErrBox("Unable to Create New Memory");
                        }
                     }
                     else
                     {
                        ErrBox("Unabled to Retrieve HTTP Response Header");
                     }
                     NetLibSocketClose(refNum,sock,SysTicksPerSecond() * 2,&neterr);
                  }
                  else
                  {
                     ErrBox("cannot write?");
                  }                     
                  break;
            }   
			}
      }
   }
   else
   {
      StrPrintF(buf,"Unabled to Create Socket, ErrorCode [%d]",neterr);
      ErrBox(buf);
   }      
   
   return -1;
   
} 

  