// Cond_ex.cpp : Defines the initialization routines and entry points for the DLL.
//

#define STRICT 1

#include <afxwin.h>         // MFC core and standard components
#include <afxext.h>         // MFC core and standard components
#include <afxcmn.h>		// for CImageList in fieldlis.h

#include <string.h> 
#include <wtypes.h>  

#include "syncmgr.h"
#include "hslog.h"
#include "condapi.h"
#include "Cond_ex.h"
#include "PALM_CMN.H"
#include "Cond_exMon.h"
#include "logstrng.h"
#include "pdcmn.h"
#include "tableDll.h"



#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static int ClientCount = 0;
static HINSTANCE hRscInstance = 0;
static HINSTANCE hDLLInstance = 0;
HINSTANCE myInst=0;
/////////////////////////////////////////////////////////////////////////////
// CCond_exApp
//

class CCond_exApp : public CWinApp
{
public:
	virtual BOOL InitInstance(); // Initialization
	virtual int ExitInstance();  // Termination 

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCond_exApp)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CCond_exApp)
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};



/////////////////////////////////////////////////////////////////////////////
// CCond_exApp

BEGIN_MESSAGE_MAP(CCond_exApp, CWinApp)
	//{{AFX_MSG_MAP(CCond_exApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
//
//       Function:            InitInstance()
//
//       Description:		basic initialisation when DLL is loaded
//
//       Parameters:		none
//                                
//       Returns:			  Success (non-zero) or Failure (zero)
//
/////////////////////////////////////////////////////////////////////////////
BOOL CCond_exApp::InitInstance()
{
	// DLL initialization 
	TRACE0("COND_EX.DLL initializing\n");

    if (!ClientCount ) {
        DWORD dwVersion;
        hDLLInstance = AfxGetInstanceHandle();

        hRscInstance = PalmLoadLanguage("COND_EXLANG.DLL", 
                       hDLLInstance, 
                       &dwVersion);

		// add any extension DLLs into CDynLinkLibrary chain
		InitTables5DLL();
    }
	myInst = hRscInstance;
    ClientCount++;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
//
//       Function:            ExitInstance()
//
//       Description:		DLL clean up before exit
//
//       Parameters:		none
//                                
//       Returns:			  Error code 
//
/////////////////////////////////////////////////////////////////////////////
int CCond_exApp::ExitInstance()
{
	TRACE0("COND_EX.DLL Terminating!\n");

	// Check for last client and clean up potential memory leak.
	if (--ClientCount <= 0) 
	{
		PalmFreeLanguage(hRscInstance, hDLLInstance);
        hRscInstance = hDLLInstance; 
	}

	// DLL clean up, if required
	return CWinApp::ExitInstance();
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CCond_exApp object

CCond_exApp conduitDll;

/////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////
//
//       Function:              OpenConduit()
//
//       Description:  Extern "C" entry point into this conduit which starts the
//                 process of synchronizing the local database table with
//                 a remote conterpart residing on the remote view device. 
//
//       Parameters:   Pointer to a callback function used to report progress.
//                                      
//                              
//
//       Returns:               
//
/////////////////////////////////////////////////////////////////////////////
ExportFunc long OpenConduit(PROGRESSFN pFn, CSyncProperties& rProps)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	long retval = -1;
	if (pFn)
	{
		CCond_exMonitor* pMonitor;

		pMonitor = new CCond_exMonitor(pFn, rProps, myInst);
		if (pMonitor)
		{
			retval = pMonitor->Engage();

			delete pMonitor;
		}
	}
	return(retval);
}

/////////////////////////////////////////////////////////////////////////////
//
//       Function:              GetConduitName()
//
//       Description:  Extern "C" entry point into this conduit which returns
//                                 the name to be used when display messages regarding
//                                 this conduit.
//
//       Parameters:   pszName - buffer in which to place the name
//                                 nLen - maximum number of bytes of buffer     
//                                      
//                              
//
//       Returns:          -1 indicates erros
//
/////////////////////////////////////////////////////////////////////////////
ExportFunc long GetConduitName(char* pszName,WORD nLen)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	char szConduitName[] = "cond_ex";

	memset(pszName, 0, nLen);
	strncpy(pszName, szConduitName, nLen-1);

	return 0;
}

/////////////////////////////////////////////////////////////////////////////
//
//       Function:     GetConduitVersion()
//
//       Description:  Extern "C" entry point into this conduit which returns
//                     the conduits version
//
//       Parameters:   none
//
//       Returns:      DWORD indicating major and minor version number
//						HIWORD - reserved
//						HIBYTE(LOWORD) - major number
//						LOBYTE(LOWORD) - minor number
//
/////////////////////////////////////////////////////////////////////////////
ExportFunc DWORD GetConduitVersion()
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return COND_EX_VERSION;
}

/////////////////////////////////////////////////////////////////////////////
//
//       Function:     ConfigureConduit
//
//       Description:  Extern "C" entry point into this conduit which returns
//                     should display the UI necessary to configure this 
//					   conduit.
//
//       Parameters:   none
//
//       Returns:      0 - success, !0 - failure
//
/////////////////////////////////////////////////////////////////////////////
ExportFunc long ConfigureConduit(CSyncPreference& pref)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	long nRtn = -1;
	// TODO - Add your configuration UI code
	return nRtn;
}

/////////////////////////////////////////////////////////////////////////////
//
//	 Method:		GetConduitInfo
//
//	 Description:	This function provides a way for a Conduit to provide info
//                  to the caller. 
//                  In this version of the call, MFC Version, Conduit Name, and
//                  Default sync action are the types of information this call
//                  will return.
//
//	 Parameters:	ConduitInfoEnum infoType - enum specifying what info is being
//                          requested.
//                  void *pInArgs - This parameter may be null, except for the Conduit
//                          name enum, this value will be a ConduitRequestInfoType structure.
//                  This following to parameters vary depending upon the info being requested. 
//                  For enum eConduitName
//                  void *pOut - will be a pointer to a character buffer
//                  DWORD *pdwOutSize - will be a pointer to a DWORD specifying the size of the character buffer.
//
//                  For enum eMfcVersion
//                  void *pOut - will be a pointer to a DWORD
//                  DWORD *pdwOutSize - will be a pointer to a DWORD specifying the size of pOut.
//
//                  For enum eDefaultAction
//                  void *pOut - will be a pointer to a eSyncType variable
//                  DWORD *pdwOutSize - will be a pointer to a DWORD specifying the size of pOut.
//
//	 Returns:		0		- Success.
//					!0      - error code.
//
/////////////////////////////////////////////////////////////////////////////
ExportFunc long GetConduitInfo(ConduitInfoEnum infoType, void *pInArgs, void *pOut, DWORD *pdwOutSize)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
    if (!pOut)
        return CONDERR_INVALID_PTR;
    if (!pdwOutSize)
        return CONDERR_INVALID_OUTSIZE_PTR;

    switch (infoType) {
        case eConduitName:
			{
            if (!pInArgs)
                return CONDERR_INVALID_INARGS_PTR;
            ConduitRequestInfoType *pInfo;
            pInfo = (ConduitRequestInfoType *)pInArgs;
            if ((pInfo->dwVersion != CONDUITREQUESTINFO_VERSION_1) ||
                (pInfo->dwSize != SZ_CONDUITREQUESTINFO))
                return CONDERR_INVALID_INARGS_STRUCT;
            
			//  Conduit name
			TCHAR szConduitName[] = "cond_ex";
			memset((TCHAR*)pOut, 0, *pdwOutSize);
			strncpy((TCHAR*)pOut, szConduitName, (*pdwOutSize)-1);
			}
            break;
        case eDefaultAction:
            if (*pdwOutSize != sizeof(eSyncTypes))
                return CONDERR_INVALID_BUFFER_SIZE;
            (*(eSyncTypes*)pOut) = eFast;
            break;
        case eMfcVersion:
            if (*pdwOutSize != sizeof(DWORD))
                return CONDERR_INVALID_BUFFER_SIZE;
            (*(DWORD*)pOut) = MFC_VERSION_50;
            break;
        default:
            return CONDERR_UNSUPPORTED_CONDUITINFO_ENUM;
    }
    return 0;
}

/////////////////////////////////////////////////////////////////////////////
//
//	 Method:		CfgConduit
//
//	 Description:	This is the second instance of ConfigureConduit. This function
//                  is provided with more info from the caller.
//                  This version of the function only supports the eConfig1 enum.
//
//	 Parameters:	ConduitCfgEnum cfgType - enum specifying the version of the 
//                      CfgConduit Call.
//                  void *pArgs - In the eConfig1 call, this is a pointer to aCfgConduitInfoType 
//                      structure. This structure provides extra info to the conduit.
//                  DWORD *pdwArgsSize - This is a pointer to a DWORD specifying the size of pArgs.
//
//	 Returns:		0		- Success.
//					!0      - error code.
//
/////////////////////////////////////////////////////////////////////////////
ExportFunc long CfgConduit( ConduitCfgEnum cfgType, void *pArgs, DWORD *pdwArgsSize)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	long nRtn = -1;

    ConduitRequestInfoType infoStruct;
    CfgConduitInfoType *pCfgInfo;

    if (!pArgs)
        return CONDERR_INVALID_INARGS_PTR;
    if (!pdwArgsSize)
        return CONDERR_INVALID_ARGSSIZE_PTR;
    if (*pdwArgsSize != SZ_CFGCONDUITINFO) 
        return CONDERR_INVALID_ARGSSIZE;

    if (cfgType != eConfig1)
            return CONDERR_UNSUPPORTED_CFGCONDUIT_ENUM;

    pCfgInfo = (CfgConduitInfoType *)pArgs;
    if (pCfgInfo->dwVersion != CFGCONDUITINFO_VERSION_1)
        return CONDERR_UNSUPPORTED_STRUCT_VERSION;



    infoStruct.dwVersion = CONDUITREQUESTINFO_VERSION_1;
    infoStruct.dwSize = SZ_CONDUITREQUESTINFO;
    infoStruct.dwCreatorId = pCfgInfo->dwCreatorId;
    infoStruct.dwUserId = pCfgInfo->dwUserId;
    strcpy(infoStruct.szUser, pCfgInfo->szUser);
    
	// TODO - Add your configuration UI code
    return nRtn;
}
