//	File:			Cond_exLink.cpp
//
//	Module:			Part of the cond_ex DLL
//
//	Description:	Source code for the DT_Link converter object. The
//					converter's job is to re-format Memo record layouts
//					from/to that of the remote Pilot's composition to that
//					of the local PC's composition. No other duties are
//					performed by this object, it is to be used soley as a
//					tool during remote communications. Its reason for existence
//					is to alieviate the main conduit code from worrying about
//					the details of remote record formats.
//
/////////////////////////////////////////////////////////////////////////////
#include "string.h"
#include "memory.h"

#include "afx.h"    // All of these includes are necessary for table stuff
#include "syncmgr.h"
#include "bfields.h"
#include "basetabl.h"
#include "Cond_ex.h"   // My own DLL's header's
#include "condapi.h"
#include "basemon.h"
#include "Cond_exTable.h"
#include "Cond_exLink.h"
#include "hslog.h"
#include "catmgr.h"

///////////////////////////////////////////////////////////////////////////
// CCond_exDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		CCond_exDTLinkConverter()      Public 
//
//	Description:	Constructor	
//
//	Parameters:		None
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
CCond_exDTLinkConverter::CCond_exDTLinkConverter(HINSTANCE hInst)
                     : CBaseDTLinkConverter(hInst)
{
}


///////////////////////////////////////////////////////////////////////////
// CCond_exDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		CCond_exDTLinkConverter()      Public 
//
//	Description:	Destructor	
//
//	Parameters:		None
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
CCond_exDTLinkConverter::~CCond_exDTLinkConverter()  
{
}


///////////////////////////////////////////////////////////////////////////
// CCond_exDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConvertToRemote()        Public 
//
//	Description:	Reformat the local PC record into the format recognized
//					by the remote device, place the results into the second
//					parameter 'rInfo'.
//
//	Parameters:	
//       CMemoRecord &rRec  - PC Record object to read from
//       CRecordInfo &rInfo - Existing memory into which is placed the record
//                            data in a layout recognized by the remote device.
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CCond_exDTLinkConverter::ConvertToRemote(CBaseRecord &rRec, 
                                           CRawRecordInfo &rInfo)
{
	long	retval = -1;
	char	*pBuff;
	int		tempInt;
	CString	tempStr;

	CCond_exRecord& rMemoRec = (CCond_exRecord &)rRec;

	retval = rMemoRec.GetRecordId(tempInt);
	rInfo.m_RecId = (long)tempInt;
	retval = rMemoRec.GetCategoryId(tempInt);
	rInfo.m_CatId = tempInt;
	
	rInfo.m_Attribs = 0;
	if (rMemoRec.IsPrivate())
		rInfo.m_Attribs |= PRIVATE_BIT;
	if (rMemoRec.IsArchived())
		rInfo.m_Attribs |= ARCHIVE_BIT;
	if (rMemoRec.IsDeleted())
		rInfo.m_Attribs |= DELETE_BIT;
	if (rMemoRec.IsModified() || rMemoRec.IsAdded())
		rInfo.m_Attribs |= DIRTY_BIT;
	
	pBuff = (char*)rInfo.m_pBytes;

	retval = rMemoRec.GetMemo(tempStr);

	char* pSrc;
	int   destLen;
	pSrc    = tempStr.GetBuffer(tempStr.GetLength());
	destLen = StripCRs(pBuff, pSrc, tempStr.GetLength());  
	tempStr.ReleaseBuffer(-1);

	rInfo.m_RecSize = destLen;

	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CCond_exDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConvertFromRemote()        Public 
//
//	Description:	Reformat the record from the remote device into the format used
//					on the local PC, place the results into the first
//					parameter 'rRec'.
//
//	Parameters:	
//		CBaseRecord& rRec		PC Record object to write into 
//		CRawRecordInfo& rInfo	Existing memory which contains the raw record
//								data from the remote device and must be parsed
//								from in order to populate the PC record object.
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CCond_exDTLinkConverter::ConvertFromRemote(CBaseRecord &rRec, 
                                             CRawRecordInfo &rInfo)
{
	long	retval = 0;
	char	*pBuff;

	CCond_exRecord& rMemoRec = (CCond_exRecord &)rRec;
	
	retval = rMemoRec.SetRecordId(rInfo.m_RecId);
	retval = rMemoRec.SetCategoryId(rInfo.m_CatId);
	if (rInfo.m_Attribs & ARCHIVE_BIT)
		retval = rMemoRec.SetArchiveBit(TRUE);
	else
		retval = rMemoRec.SetArchiveBit(FALSE); 

	if (rInfo.m_Attribs & PRIVATE_BIT)
		retval = rMemoRec.SetPrivate(TRUE);
	else
		retval = rMemoRec.SetPrivate(FALSE); 
	
	retval = rMemoRec.SetStatus(fldStatusNONE);
	if (rInfo.m_Attribs & DELETE_BIT) // Delete flag
		retval = rMemoRec.SetStatus(fldStatusDELETE);
	else if (rInfo.m_Attribs & DIRTY_BIT) // Dirty flag
		retval = rMemoRec.SetStatus(fldStatusUPDATE);

	// Only convert body if remote record is *not* deleted..
	if (!(rInfo.m_Attribs & DELETE_BIT)) 
	{
		pBuff = (char*)rInfo.m_pBytes;

		// Add in any necessary CRs, result is placed in m_TransBuff
		AddCRs(pBuff, strlen(pBuff));
		CString aString(m_TransBuff);
		retval = rMemoRec.SetMemo(aString);
	}

	return(retval);
}

///////////////////////////////////////////////////////////////////////////
// CCond_exDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConvertFromRemoteCategories() 
//
//	Description:	Offset into the specific appInfoBlock to the spot where
//					my category labels begin.  Then, call base converter to
//					do the extraction. If the passed in DbGenInfo has *no* 
//					bytes allocated onto the m_pBytes pointer, then return
//					with an error indicating no AppInfo raw bytes were present
//					to parse from.
//
//	Parameters:		CDbGenInfo&
//					CCategoryMgr*
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CCond_exDTLinkConverter::ConvertFromRemoteCategories(CDbGenInfo& dbInfo, 
                                                       CCategoryMgr* catMgr)
{
	long retval = CONDERR_CONVERT_TO_LOCAL_CATS;
	char* pBuff;
	short wTemp;

	if (dbInfo.m_pBytes)
	{
		pBuff = (char*)dbInfo.m_pBytes;

		// Now 'walk' over the un-important bytes until we reach the
		// offset where the category labels begin.  This walk will be
		// be different for each appInfo block.
		
		// Get the dirty category flags
		wTemp = *((WORD*)pBuff);
		wTemp = FlipWord(wTemp);

		pBuff += sizeof(WORD);   // dirtyCategories is in front of Categories
		retval = CBaseDTLinkConverter::ExtractCategories(pBuff, wTemp, catMgr);		
	}
	return(retval);
}


///////////////////////////////////////////////////////////////////////////
// CCond_exDTLinkConverter 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConvertToRemoteCategories() 
//
//	Description:	Offset into the specific appInfoBlock to the spot where
//					my category labels begin.  Then, call base converter to
//					do the replacement. If the passed in DbGenInfo has *no* 
//					bytes allocated onto the m_pBytes pointer, then return
//					with an error indicating no AppInfo raw bytes were present
//					to parse from.
//
//	Parameters:		CDbGenInfo&
//					CCategoryMgr*
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CCond_exDTLinkConverter::ConvertToRemoteCategories(CDbGenInfo& dbInfo, 
                                                     CCategoryMgr* catMgr)
{
	long retval = CONDERR_CONVERT_TO_REMOTE_CATS;
	char* pBuff;

	if (dbInfo.m_pBytes)
	{
		pBuff = (char*)dbInfo.m_pBytes;

		// Now 'walk' over the un-important bytes until we reach the
		// offset where the category labels begin.. This walk will be
		// be different for each appInfo block.

		*((WORD *)pBuff) = 0;	// Clear the category dirty flags
		pBuff += sizeof(WORD);
		retval = CBaseDTLinkConverter::ReplaceCategories(pBuff, catMgr);
	}
	return(retval);
}


