//	File:			Cond_exMon.h
//
//	Module:			Part of the cond_ex DLL
//
//	Description:	The 'CCond_exMonitor' class which controls the flow
//					of synchronization 
/////////////////////////////////////////////////////////////////////////////

#include "string.h"
#include "memory.h"

#include "afx.h"    // All of these includes are necessary for table stuff
#include "bfields.h"
#include "basetabl.h"
#include "Cond_exTable.h"

#include "syncmgr.h"
#include "condapi.h"
#include "hslog.h"
#include "basemon.h"
#include "Cond_ex.h"   // My own DLL's header's
#include "Cond_exMon.h"
#include "Cond_exLink.h"
#include "updcatid.h"
#include "catmgr.h"
#include "logstrng.h"   // for Resource strings

#include "stdafx.h"

#define	ARCHIVE_FILE_EXT	".MPA"

// memos greater than this will get split into multiple memos
#define LARGE_TEXT_LIMIT	4096    
#define MEMO_TITLE_LENGTH   35     // in bytes for large memos 


///////////////////////////////////////////////////////////////////////////
// CCond_exMonitor 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		CCond_exMonitor()      Public 
//
//	Description:	Constructor	
//
//	Parameters:		PROGRESSFN			Used for callback to send progress
//					CSyncProperties&	Contains syncType and filenames
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
CCond_exMonitor::CCond_exMonitor(PROGRESSFN pFn,
                                               CSyncProperties& rProps, 
                                               HINSTANCE hInst)
                    : CBaseConduitMonitor(pFn, rProps, hInst)
{
	m_pDTConvert = new CCond_exDTLinkConverter(hInst);
	m_wRawRecSize = COND_EX_RAW_REC_MEM;
	m_TotRemoteDBs = 1;
	m_CurrRemoteDB = 0;
}

///////////////////////////////////////////////////////////////////////////
//
//	Function:		CCond_exMonitor()      Public 
//
//	Description:	Destructor	
//
//	Parameters:		None
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
CCond_exMonitor::~CCond_exMonitor()  
{
}

///////////////////////////////////////////////////////////////////////////
//
//	Function:		EngageStandard()       
//
//	Description:		
//
//	Parameters:		None
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
long CCond_exMonitor::EngageStandard()  
{
	// upload data from handheld device
	m_rSyncProperties.m_SyncType = eHHtoPC;
	return CBaseConduitMonitor::EngageStandard();
}

 
///////////////////////////////////////////////////////////////////////////
//
//	Function:		ApplyRemotePositionMap()      ** virtual Protected ** 
//
//	Description:    Override to diffuse activity.	
//
//	Parameters:		None
//
//	Returns:		n/a
//
///////////////////////////////////////////////////////////////////////////
long CCond_exMonitor::ApplyRemotePositionMap(void) 
{
	long 			retval=0, lErr, i=0;	
	CPositionInfo	posInfo;
	long 			*pLongs;
	long            numRecs = 0;

	memset(&posInfo, 0, sizeof(posInfo));

	// Alloc enough memory for all the record Id's
	if (m_LocRealTable) 
		numRecs = m_LocRealTable->GetRecordCount();

	if (numRecs > 0)
	{
		posInfo.m_TotalBytes = numRecs * sizeof(long);
		posInfo.m_pBytes     = (BYTE*) new char [posInfo.m_TotalBytes];
		posInfo.m_FileHandle = m_RemHandle;
		posInfo.m_MaxEntries = (WORD)numRecs;  

		// Read in the remote array of record Id's (motorola format)
		retval = SyncReadPositionXMap(posInfo);

		if (!retval)
		{
			if (!m_pDTConvert->ConvertPositionMap(posInfo))
			{
				pLongs = (long*)posInfo.m_pBytes;
	
				m_LocRealTable->ClearPlacementField();
	
				CBaseRecord     locRec(*m_LocRealTable);
				CBaseIterator   locIt(*m_LocRealTable);
				locIt.SortByRecordId();
		
				lErr = locIt.FindFirst(locRec, FALSE);
				while (!lErr && i < posInfo.m_NumReadIn)
				{
					if (!locIt.FindByRecordId(pLongs[i], locRec, FALSE))
					{
						locRec.SetPlacementPos(i);
					}
					i++;
				}
			}
		}
		if (posInfo.m_pBytes) 
			delete posInfo.m_pBytes;     

	}
	return(retval);
}

///////////////////////////////////////////////////////////////////////////
//
//	Function:		CreateTable()      ** Protected **     
//
//	Description:	Create a specific table for the base function.	
//
//	Parameters:
//			CBaseTable* pBase - gets (Table object) memory allocated to it
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CCond_exMonitor::CreateTable(CBaseTable*& pBase)
{
	long retval = -1;

	pBase = new CCond_exTable();
	if (pBase)
		retval = 0;

	return(retval);
}

///////////////////////////////////////////////////////////////////////////
//
//	Function:		ConstructRecord()      ** Protected **     
//
//	Description:	Create a derived class record object to be used in base
//					class functions 
//
//	Parameters:
//			CBaseRecord* pBase - pointer to base class record that will 
//								 point to derived class object
//			CBaseTable& rtable - table associated with the record, reqd 
//								 for constructing Record
//			WORD wModAction	   - reqd for constructing Record
//
//	Returns:		long	error code
//
///////////////////////////////////////////////////////////////////////////
long CCond_exMonitor::ConstructRecord(CBaseRecord*& pBase, 
											 CBaseTable& rtable ,
											 WORD wModAction)
{
	long retval = -1;

	pBase = new CCond_exRecord(rtable, wModAction);

	if (pBase)
		retval = 0;

	return(retval);
}

///////////////////////////////////////////////////////////////////////////
//
//	Function:		SetArchiveFileExt()      ** Protected **     
//
//	Description:	Set the archive file extension for this conduit
//					 
//	Parameters:		none
//			
//
//	Returns:		void
//
///////////////////////////////////////////////////////////////////////////
void CCond_exMonitor::SetArchiveFileExt()
{
	strcpy(m_ArchFileExt, ARCHIVE_FILE_EXT);
}


///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogRecordData()      ** Protected **     
// 
//
//	Description:	Function to assist in constructing meaningful log
//					entries during the troubles that may occur during
//					a sync session. 
//
//                  e.g - For the Address Conduit, provide the text form
//                  of the 3 fields: Name, FirstName, & Company.
//
//	Parameters:		CBaseRecord&  - record object causing possible error	
//					char* errBuff - buffer to place some field values into
//
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CCond_exMonitor::LogRecordData(CBaseRecord& rRec, char * errBuff)
{
	CCond_exRecord	&rLocRec = (CCond_exRecord&)rRec;
	CString     csNote;
	int         noteLen = 0;

	rLocRec.GetMemo(csNote);
	noteLen = csNote.GetLength();
	if (noteLen > 70)
		noteLen = 70;

	strcpy(errBuff, "       ");
	strncat(errBuff, csNote, noteLen);
}

///////////////////////////////////////////////////////////////////////////
//
//	Function:		LogApplicationName()      ** Protected **     
// 
//	Description:	Returns the name of the application.
//
//	Parameters:		char* appName - buffer to place the name into
//					WORD len - length of appName
//
//	Returns:		Nothing
//
///////////////////////////////////////////////////////////////////////////
void CCond_exMonitor::LogApplicationName(char* appName, WORD len)
{
	char szConduitName[] = "cond_ex";

	memset(appName, 0, len);
	strncpy(appName, szConduitName, len-1);

}


